classdef AcmeSim < handle
% ACMESIM Object to hold all the simulation information
%   AcmeSim contains parameters of an ACME simulation and post-processing.
% The initialization will create the frequency and time arrays assuming a fixed
% step, although the actual simulation time vector will depend on the solver.
%
%%% Properties:
%
% * modelName   - name of an .slx file; 
% * modelHandle - index of a .slx system loaded into memore;
% * stopTime    - sim duration, s;
% * nSteps      - number of steps (assumes fixed step);
% * frequencies - logspaced vector of frequencies between Nyquist frequency
%  and minimum frequency (maximum period);
% * stepSize    - step size (assumes fixed step), s;
% * sampleFrequency - sample frequency; 
% * time        - equally spaced time vector, from 0 to stopTime;
% * ENBW        - Equivalent Noise Bandwidth;
% * eps0 = 8.86e-12 F/m, vacuum permittivity;
% * kb   = 1.3806e-23 boltzmann;
% * T = 300  K;
% * flpsdOpts   - parameters to pass to flpsd;
% * blocksHandles - array of handles of the blocks contained in the module.
% 
%%% Methods:
%								
% * AcmeSim - class constructor.
%	  * obj = AcmeSim(ModelName, StopTime, NumberOfSteps, flpsdOpts);
% 	  * Inputs: 
% 		  * ModelName - namepath of valid .slx file. Required;
% 		  * StopTime  - sim duration, s. Required;
% 		  * NumberOfSteps - number of steps, s. Required;
% 		  * flpsdOpts     - paramaters to pass ot flpsd for post-processing.
% Optional. Will default to AcmeSim.flpsdOptions;
% 
% * openModel - opens Simulink with the loaded acme model.
% 	* openModel(obj)
% 
% * addComponent - base function to add Acme component to this Acme instance.
% 	* cmpnt = addComponent(obj, componentType, varargin);
% 	* Private function.
% 	* Inputs:
% 		* componentType - type of component;
% 		* varargin - parameters to be passed to specific add block function;
% 	* Outputs:
% 		* cmpnt - Component object;
% 
% * addElectrode - Add component Electrode to this Acme instance.
% 	* el = addElectrode(obj, BlockName, LengthSides, pos0, varargin);
% 	* Deprecated. Use S-Function version.
% 	* Inputs:
% 		* BlockName - name of the block in the simulink model, char array;
% 		* LengthSides - length of the sides of the electrode, m, (1,2) double;
% 		* pos0 - initial position of the electrode, m, i.r.t. CoM (1,3)
% double;
% 		* varargin - 
% 			* V_DC - DC bias of the electrode, V;
% 			* V_AC - AC bias of the electrode, Vrms;
% 	* Output:
% 		* el - Electrode object;
% 
% * addCapacitor - Add component Capacitor to this Acme instance.
% 	* cp = addCapacitor(obj, BlockName, varargin)
% 	* Deprecated. Use S-Function version.
% 	* Inputs:
% 		* BlockName - name of the block in the simulink model, char array
% 		* varargin - 
% 			* eps0 - dielectric constant, default eps0 = 8.854e-12 F/m
% 	* Outputs:
% 		* cp - Capacitor object
% 
% * addTestMass - Add component TestMass to this Acme instance.
% 	* tm = addTestMass(obj, BlockName, lengthSides, gap, varargin)
% 	* Input:
% 		* BlockName - name of the block in the simulink model, char array
% 		* LengthSides - length of the sides of the electrode, in meters,
% 1x2 double
% 		* pos0 - initial position of the electrode in along the DoF, in
% 	 meters, i.r.t. CoM
% 		* varargin - 
% 			* V_DC - DC bias of the electrode, V
% 			* V_AC - AC bias of the electrode, Vrms
% 	* Output:
% 		* el - Electrode object
% 
% * addCapacitorSF - add Capacitor (S-Function) to this Acme instance.
% 	* cp = addCapacitorSF(obj, BlockName, varargin)
% 	* Inputs:
% 		* BlockName - name of the block in the simulink model, char array
% 		* varargin - 
% 			* eps0 - dielectric constant, default eps0 = 8.854e-12 F/m
% 	* Output:
% 		* cp - Capacitor object
% 
% * addElectrodeSF - Add Electrode (S-Function) to this Acme instance.
% 	* el = addElectrodeSF(obj, BlockName, LengthSides, pos0, varargin);
% 	* Inputs:
% 		* BlockName - name of the block in the simulink model, char array;
% 		* LengthSides - length of the sides of the electrode, m, (1,2) double;
% 		* pos0 - initial position of the electrode, m, i.r.t. CoM (1,3)
% 		double;
% 		* varargin - 
% 			* V_DC - DC bias of the electrode, V;
% 			* V_AC - AC bias of the electrode, Vrms;
% 	* Output:
% 		* el - Electrode object;
% 
%%% 
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 11.08.2023;

properties
    modelName 
    modelHandle  (1,1) double
    stopTime     (1,1) double
    nSteps       (1,1) uint32
    frequencies        double
    stepSize     (1,1) double
    sampleFrequency  (1,1) double
    time
    ENBW 
    eps0 = 8.86e-12; %F/m, vacuum permittivity
    kb = 1.3806e-23; %boltzmann
    T = 300; % K
    flpsdOpts
    blocksHandles
end

methods
    function obj = AcmeSim(varargin)
        %ACMESIM Construct an instance of this class
        flpsdopts = {'fs',        1,...
                     'Lmin' ,     1,...
                     'order',     0,...
                     'olap' ,    30,...             
                     'Kdes' ,   100,...
                     'Jdes' ,  1000,...
                     'scale', 'ASD',...
                     'win'  , 'hanning',...
                     'LTPDAmode',false,...                         
                    };
        
        p = inputParser;
        p.addRequired('ModelName',    @(x) ischar(x)||isstring(x));
        p.addRequired('StopTime',     @(x) isscalar(x));
        p.addRequired('NumberOfSteps',@(x) isscalar(x));
        p.addOptional('flpsdOpts', flpsdopts);
        p.parse(varargin{:});

        obj.modelName     = p.Results.ModelName;
        obj.stopTime      = p.Results.StopTime;
        obj.nSteps = p.Results.NumberOfSteps;
        obj.flpsdOpts     = p.Results.flpsdOpts;
        obj.ENBW          = 1/obj.stopTime;
        obj.stepSize      = obj.stopTime/obj.nSteps;
        obj.time          = (0:obj.stepSize:(obj.stopTime-obj.stepSize))';
        obj.sampleFrequency = 1/obj.stepSize;   
        obj.frequencies   = logspace(-log10(obj.stopTime),log10(obj.sampleFrequency/2),100);    
        obj.flpsdOpts{2}  = obj.sampleFrequency;
        
        % Load simulink model
        obj.modelHandle = load_system(obj.modelName);
        obj.blocksHandles = [];
    end

    function cmpnt = addComponent(obj, componentType, varargin)
        % Wrapper to include the model name
        model = obj.modelName;
        solver = "Variable-step";
        
        % calls the right constructor
        switch lower(componentType)
            %case 'electrode'
           %     Electrode(model, block, area, pos0, V_DC, V_AC, f)
              %  cmpnt = Electrode(varargin{:},'model',model);
            case 'electrode'
                cmpnt = Electrode(varargin{:},'model',model);
			case 'electrode222'
                cmpnt = ElectrodeHW222(varargin{:},'model',model);
            case 'capacitor'
                %cmpnt = Capacitor(varargin{:},'model',model);
            case 'capacitorsf'
                cmpnt = Capacitor(varargin{:},'model',model);
            case 'testmass'
                cmpnt = TestMass(varargin{:},'model',model);
            otherwise
                error('No component match')
        end
        obj.blocksHandles = [obj.blocksHandles cmpnt.componentBlockHandle];
    end

    function tm = addTestMass(obj, BlockName, LengthSides, gap, varargin)

        % TO DO: add to the list of known blocks in acme?

        tm = addComponent(obj,'testmass',LengthSides, gap,...% voltage,...%electrodeA
            'BlockName', BlockName, varargin{:});
    end
    
    function el = addElectrode(obj, BlockName, LengthSides, pos0, V_DC,...
            channels, V_ACs)
        % Add component Electrode from S-Function to acme
        % Syntax: 
        %    el = addElectrodeSF()
        % Input:
        %    BlockName - name of the block in the simulink model, char array
        %    LengthSides - 
        %    pos0 - 
        % Output:
        %    el - Electrode object
        %
        % Author: Arthur Reis
        % email: arthur.reis@aei.mpg.de
        % Last revision: 


        el = addComponent(obj,'electrode', 'BlockName', BlockName, ...
                          'LengthSides', LengthSides, 'Position', pos0, ...
                          'V_DC', V_DC, 'V_ACs', V_ACs, 'Channels', channels);
    end
    
	function el = addElectrode222(obj, BlockName, TM)
        % Add component Electrode222 from S-Function to acme
        % Syntax: 
        %    el = addElectrode222()
        % Input:
        %    BlockName - name of the block in the simulink model, char array
        %    LengthSides - 
        %    pos0 - 
        % Output:
        %    el - Electrode object
        %
        % Author: Arthur Reis
        % email: arthur.reis@aei.mpg.de
        % Last revision: 
		TM_sides

        el = addComponent(obj,'electrode', 'BlockName', BlockName, ...
                          'TM', TM);
    end

    function cp = addCapacitor(obj, BlockName, varargin)
        p = inputParser;
        %p.addOptional('ElectrodePair',['','']);
        p.addOptional('Electrode','');
        p.addOptional('ElectrodeOffset', [0 0 0]);
        p.addOptional('ForceNormal', [0 0 0]);
        p.addOptional('Eps0', obj.eps0);
        
        p.parse(varargin{:});

        eps0 = p.Results.Eps0; %#ok<PROPLC> 
        if eps0 == ''
            eps0 = obj.eps0;
        end
        % TO DO: add to the list of known blocks in acme?
        if p.Results.ForceNormal == [0 0 0]
           normal = p.Results.Electrode.normal;
        else
            normal = p.Results.ForceNormal;
        end    
        cp = addComponent(obj,'capacitorsf', 'BlockName', BlockName,...
            'Eps0', eps0, 'Electrode',p.Results.Electrode, ...
            'ElectrodeOffset', p.Results.ElectrodeOffset,...
            'ForceNormal',normal); %#ok<PROPLC> 
    end

    function n = noiseTS(obj,noiseASD, varargin)
        outputmode = 'array';
        if length(varargin) == 1
            outputmode = varargin{1};
        end
        if isa(noiseASD,"function_handle")
            nTS = timeseriesFromASD(noiseASD, obj.ENBW,...
                obj.sampleFrequency, obj.nSteps);  
        elseif isa(noiseASD,"double")
            nTS = timeseriesFromFrequencyArray(noiseASD,obj);
        end
        switch lower(outputmode)
            case 'ts'
                n = nTS;
            case 'data_only'
                n = nTS.Data;
            case 'array'
                n = [obj.time nTS.Data];
        end
            
    end

    function o = openModel(obj)
        o = open(obj.modelName);
    end

	function p = get_param(obj,parameter)
		p = get_param(obj.modelName,parameter );
	end

	function checkBlocks(obj)
		bl = get_param(obj.modelName, 'blocks');
		for i=1:length(bl)
			btype = get_param([obj.modelName '/' bl{i}], 'blockType');
				if strcmp(btype, 'M-S-Function')
					switch get_param([obj.modelName '/' bl{i}], 'FunctionName')
					case 'electrodesf' % check electrode blocks
						%
		
					case 'capacitorsf' % check capacitor blocks
						%

					case 'sumforcesf'
						% create mask
						bh = get_param([obj.modelName '/' bl{i}], 'Handle');
						bp = get_param([obj.modelName '/' bl{i}], 'Parameters');
						sumforcemasking(bh, str2num(bp));
					case 'sumtorquesf'
						% create mask
						bh = get_param([obj.modelName '/' bl{i}], 'Handle');
						bp = get_param([obj.modelName '/' bl{i}], 'Parameters');
						sumtorquemasking(bh, str2num(bp));

					otherwise
						%throw (MException('Simulink:WrongBlockType',""));		
					end
					% check tm blocks
					
					% check capacitor blocks
					% check sumation blocks

				end
		end
	end
end
% methods(Static)
%     function s = flpsdOptions()
% 
%     end
% end
end

