classdef Capacitor < Component
%CAPACITORSF Wrapper class for the Capacitor s-function block 
%   This class creates an object that collects all the utilities around and 
% connects to a copy of the capacitor s-function block.
%
%%% Properties:
%
% * componentBlockName - same as superclass
% * componentBlockHandle - same as superclass
% * attachedModel - same as superclass
% * updateFlag - same as superclass
% * blockSetParameters - parameters passed to block mask during setup 
% (nChannels, pos0, area, area, V_DC);
% * blockUpdateParameters - parameters passed to block mask during update
%  (nChannels, pos0, area, area, V_DC);
% * blockMask - name of the corresponding S-Function block
% * electrode - Electrode instance that pairs with the test mass
% * eps0 - dielectric constant
% * channels - array of strings with name of Control signals (DoFs);
% * electrodeOffset - position were the point force is calculated on the TM
% * forceNormal - direction vector of the force;
% * gap0 - nominal gap between electrode and TM;
% * c0 - nominal capacitance at rest position
% * area - effective area of the capacitor (= area of the electrode)
%
%%% Methods:
%
% * Capacitor - object constructor. Should be added to an acme instance through
% addCapacitor call. Example:
% `cp = acmeinstance.addCapacitor(BlockName, Electrode, ElectrodeOffset)`
%     * Output:
%         * cp - Electrode object. 
%     * Inputs:   
%         * BlockName - char. Name of the block in the Simulink model. Must be a
%         Matlab Level 2 "capacitorsf" S-Function.
%         * Electrode - Electrode instance that pairs with the test mass
%         * ElectrodeOffset - position were the point force is calculated 
% on the TM;
% 
% * force - outputs a force (f, in N) for a given voltage (V, in V), considering
% the nominal capacitance.
%		`f = force(obj, V)`
% 
% * voltage_from_force - calculates the voltage required 
% * force_from_CPD - estimate the contribution of contact point differential
% force
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 16.08.2023;

properties
    electrode
    eps0
    channels 
    electrodeOffset
    forceNormal
	forceModel
end

properties (Dependent)
    gap0
    c0
    area
	blockSetParameters
	blockUpdateParameters
	blockMask
end
    
methods
    function obj = Capacitor(varargin)
        p = inputParser;
        p.addParameter('Model', '', @(x) ischar(x));
        p.addParameter('BlockName', '', @(x) ischar(x));
        p.addParameter('Electrode','')
        p.addParameter('ElectrodeOffset','')
        p.addParameter('ForceNormal','')
        p.addParameter('Channels',[""]);            
        p.addParameter('Eps0',  '');
        p.parse(varargin{:});
        
        obj.eps0 = p.Results.Eps0;
        obj.attachedModel = p.Results.Model;
        obj.componentBlockName = p.Results.BlockName;
        obj.componentBlockHandle = obj.getHandle;
		obj.checkBlock;
        obj.electrode = p.Results.Electrode;
        %obj.electrodeB = p.Results.ElectrodePair(2);
        obj.electrodeOffset = p.Results.ElectrodeOffset;
        obj.forceNormal     = p.Results.ForceNormal;
       % obj.forceModel = 2; %simplified
		obj.channels   = obj.electrode.channels;
        obj.updateFlag = true;

        set_param(obj.componentBlockHandle,'linkstatus','none');
        % break library links so masks are editable
		obj.setBlock()		
		obj.setMask()	
    end
    
    function set.eps0(obj, value)
        obj.eps0 = value;
        obj.updateBlock;
    end

    function set.channels(obj, value)
        obj.channels = value;
        obj.updateBlock;
    end

    function g = get.gap0(obj)
        g = abs(norm(obj.electrode.pos0 - obj.electrodeOffset));
        
    end

    function c = get.c0(obj)
        c    = obj.eps0 * obj.area/obj.gap0;
    end

    function a = get.area(obj)
        a = obj.electrode.area;
    end

    function f = force(obj, V)
        f = obj.c0 * V^2 /(2*obj.gap0);
    end

    function v = voltage_from_force(obj, f)
	%	why?
        v = sqrt(f * 2* obj.gap0/obj.c0 );
    end

    function F = force_from_CPD(obj, varargin)
        % see Speake & Trenkel 2003
        varvoltrms = 100e-3;
        
        if length(varargin) == 1
            varvoltrms = varargin{1};
        end

        kpeak = 1.3/obj.gap0;
        kmax = 1.2*kpeak;
        kmin = 0.8*kpeak;

        func = @(k) k.^3 ./(sinh(obj.gap0.*k)).^2;
        intr = integral(func,kmin,kmax);

        fp = -2*obj.eps0*(varvoltrms)^2*intr/(kmax^2 - kmin^2);
        F = abs(fp*obj.area);
    end

	function checkBlock(obj)
		if ~strcmp(get_param(obj.componentBlockHandle,'Parameters'), ...
				'nChannels, eps0, posOffset, forceNormal')
			throw(MException('Simulink:WrongSFunctionParms',"Wrong" + ...
				" S-Function parameters."));
		end

		if ~strcmp(get_param(obj.componentBlockHandle,'BlockType'), ...
				'M-S-Function')||...
		   ~strcmp(get_param(obj.componentBlockHandle,'FunctionName'),...
				'capacitorsf')
			throw(MException('Simulink:WrongBlockType',"Electrode should be a" + ...
				" level 2 s-function block with 'capacitorsf' function."));		
		end
		return
	end

	function P = get.blockSetParameters(obj)
		pn{1} = 'nChannels';
		pn{2} = 'eps0';
		pn{3} = 'posOffset';
		pn{4} = 'forceNormal';
		%pn{5} = 'forceModel';
		%pn{5} = 'gap0';
		pv{1} = length(obj.channels);
		pv{2} = obj.eps0;
		pv{3} = obj.electrodeOffset;
		pv{4} = obj.forceNormal;
		%pv{5} = obj.forceModel;
		%pv{5} = obj.gap0;
		P = {pn, pv};
	end

	function P = get.blockUpdateParameters(obj)
		pn{1} = 'nChannels';
		pn{2} = 'eps0';
		pn{3} = 'posOffset';
		pn{4} = 'forceNormal';
		pv{1} = length(obj.channels);
		pv{2} = obj.eps0;
		pv{3} = obj.electrodeOffset;
		pv{4} = obj.forceNormal;
		P = {pn, pv};
	end

	function M = get.blockMask(obj)
% dsp = strcat(dsp, "port_label('output', 1, 'C');");
% dsp = strcat(dsp, "port_label('output', 2, 'gap');");
% dsp = strcat(dsp, "port_label('output', 3, 'pos');");
% for i=1:length(channels)
%     dsp = strcat(dsp, "port_label('input',", ...
%           num2str(5+i), ", 'V",channels(i),"');");
%     dsp = strcat(dsp, "port_label('output',",  num2str(3+i), ...
%         ", 'F",channels(i),"');");
% end
% 
% m.Display = dsp;
% set_param(blockHandle,"MaskSelfModifiable",'on');
% set_param(blockHandle,"MaskObject",m);
		M = "image('planeCapTMEL.png');" + newline;
		M = strcat(M, "port_label('input',  1, 'TM pos');" + newline);
		M = strcat(M, "port_label('input',  2, 'TM rot');" + newline);
		M = strcat(M, "port_label('input',  3, 'V_TM');"   + newline);
		M = strcat(M, "port_label('input',  4, 'El pos');" + newline);
		M = strcat(M, "port_label('input',  5, 'A');" + newline);
		M = strcat(M, "port_label('input',  6, 'VDC');" + newline);
 
		M = strcat(M, "port_label('output', 1, 'C');" + newline);
		M = strcat(M, "port_label('output', 2, 'gap');" + newline);
		M = strcat(M, "port_label('output', 3, 'pos');" + newline);
		
		for i=1:length(obj.channels)
			M = strcat(M, "port_label('input',", ...
			num2str(6+i), ", 'V",obj.channels(i),"');");
			M = strcat(M, "port_label('output',",  num2str(3+i), ...
				", 'F",obj.channels(i),"');");
		end
		
		%m.Display = dsp;
		%set_param(blockHandle,"MaskSelfModifiable",'on');
		%set_param(blockHandle,"MaskObject",m);

	end

	function K = k(obj, Vd)
		K = -2*obj.area*obj.eps0*Vd^2/obj.gap0^3;
	end
end

end