classdef  Component < handle
%COMPONENT Superclass of all acme components classes
%   This superclass has all the properties and methods common to any acme
%   component classes. Inherits the "handle" superclass, that define 
%   objects that reference the object so copying an object creates another
%   reference to the same object.
%
%%% Properties:
%
% * componentBlockName   - name of the block in the model;
% * componentBlockHandle - handle of the block in the model;
% * attachedModel - name of the model;
% * updateFlag    - boolean to stop updating block before object is ready;
% * blockSetParameters    - cell 'dict' of parameters k/v to set up block;
% * blockUpdateParameters - cell 'dict' of parameters k/v to update block;
% * blockMask -
%
%%% Methods:
%
% * getHandle - returns the handle of the block in the loaded Simulink system
%  
% * setBlock - passes the parameter key-value pairs in blockSetParameters to the
%block mask in Simulink
%
% * setMask -
%
% * updateBlock - passes the parameter key-value pairs in blockUpdateParameters
%  to the block mask in Simulink
%
% * get_param - overload of Simulink get_param()
%
% * set_param - overload of Simulink set_param()
%
% * checkBlock - abstract
%
%%%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 14.08.2023;

properties 
    componentBlockName char
    componentBlockHandle double
    attachedModel char
    updateFlag (1,1) logical = false
end

properties(Abstract)
	blockSetParameters
	blockUpdateParameters
	blockMask
end

methods 
    function hd = getHandle(obj)
        hd = Simulink.findBlocks(obj.attachedModel, ...
            'Name',obj.componentBlockName);
        if isempty(hd)
            error("Block not found");
        end
    end
	
	function setBlock(obj)
		parms = obj.blockSetParameters;
		for i = 1:length(parms{1})
			parmName = parms{1}{i};
			parmValue = parms{2}{i};
			obj.set_param(parmName, parmValue);
		end
	end
	
	function setMask(obj)
		m = get_param(obj.componentBlockHandle,"MaskObject");
		m.Display = obj.blockMask;
		set_param(obj.componentBlockHandle,"MaskSelfModifiable",'on');
		set_param(obj.componentBlockHandle,"MaskObject",m);
	end

	function updateBlock(obj)
		if obj.updateFlag
			parms = obj.blockUpdateParameters;
			for i = 1:length(parms{1})
				parmName = parms{1}{i};
				parmValue = parms{2}{i};
				obj.set_param(parmName, parmValue);
			end
		end
	end

	function p = get_param(obj,parameter)
		p = get_param([obj.attachedModel '/' obj.componentBlockName], ...
			parameter );
	end

	function set_param(obj,parameter,value)
		if isscalar(value)
			set_param([obj.attachedModel '/' obj.componentBlockName], ...
			parameter, num2str(value));
		elseif isnumeric(value)
			set_param([obj.attachedModel '/' obj.componentBlockName], ...
			parameter, mat2str(value));
		elseif iscell(value)
			for i=1:length(value)
				set_param([obj.attachedModel '/' obj.componentBlockName], ...
				parameter{i}, value{i})
			end
		elseif ischar(value)	
			set_param([obj.attachedModel '/' obj.componentBlockName], ...
			parameter, value)
		end
		
		%set_param([obj.attachedModel '/' obj.componentBlockName], ...
		%	parameter, value)
	end
	
end

methods(Abstract)
	checkBlock(obj)
end

end

