classdef Electrode < Component
%ELECTRODE Wrapper class for the electrode S-function block 
%   This class creates an object that collects all the utilities around and 
% connects to a copy of the electrode s-function block.
%
%%% Properties:
%
% * componentBlockName - same as superclass
% * componentBlockHandle - same as superclass
% * attachedModel - same as superclass
% * updateFlag - same as superclass
% * blockSetParameters - parameters passed to block mask during setup 
% (nChannels, pos0, area, area, V_DC);
% * blockUpdateParameters - parameters passed to block mask during update
%  (nChannels, pos0, area, area, V_DC);
% * blockMask - 
% * pos0 - position in relation to x, y, z initial TM position, m
% * sides - electrode size in x, y, z (has to have 2 non-zero dimensions), m
% * V_DC - (1 x nChannels) V DC bias, V.
% * V_ACs - TODO: should this exist?
% * channels - array of strings with name of Control signals (DoFs);
% * area - area of the electrode, m2
% * normal - normalized vector in x, y, z perpendicular to the electrode,
% towards the TM;
% 
%%% Methods:
%
% * Electrode - object constructor. Should be added to an acme instance through
% addElectrode call. Example:
% el = acmeinstance.addElectrode(BlockName, LengthSides, pos0, V_DC, Channels,
%  V_ACs)
%     Output:
%         * el - Electrode object. 
%     Inputs:   
%         * BlockName - char. Name of the block in the Simulink model. Must be a
%         Matlab Level 2 "electrodesf" S-Function.
%         * Position - (1x3) double, m. Initial TM position.
%         * LengthSides - (1x3) double, m. Must have exactly 2 non-zero values.
%         * V_DC - DC bias, V.
%         * Channels - (1 x n) string. Name of the DoFs.
%         * V_ACs - AC bias, Vrms
% 
% * checkBlock - checks if the block has the right s-function and mask
% parameters;
%
%%%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 13.06.2023;

properties
    pos0    (1,3) double % position in relation to initial TM position
    sides   (1,3) double % electrode size  
    V_DC    (1,1) double % DC bias. Is added to all channels!
    V_ACs                % TODO: should this exist?
    channels             % Control signals (DoFs)
end

properties (Dependent)
    area   (1,1) double  %
    normal (1,3) double  % unitary vector to get the force direction
	blockSetParameters
	blockUpdateParameters
	blockMask
end

methods
    function obj = Electrode(varargin)        
        p = inputParser;
        p.addParameter('BlockName', '', @(x) ischar(x));
        p.addParameter('Model', '', @(x) ischar(x));
        p.addParameter('Position', [0 0 0]);
        p.addParameter('LengthSides',[0 0]);
        p.addParameter('V_DC', 0);
        p.addParameter('V_ACs', []);
        p.addParameter('Channels',[""]);
        p.parse(varargin{:})
        
        blockName = p.Results.BlockName;
        model     = p.Results.Model;
        pos0      = p.Results.Position;
        sides     = p.Results.LengthSides;
        V_DC      = p.Results.V_DC;
        V_ACs     = p.Results.V_ACs;
        channels  = p.Results.Channels;
        
        obj.componentBlockName = blockName;
        obj.attachedModel = model;
        obj.componentBlockHandle = obj.getHandle;

		obj.checkBlock;
        obj.pos0 = pos0;
        obj.sides = sides;
        obj.V_ACs = V_ACs;
        obj.V_DC = V_DC;
        obj.channels = channels;
		obj.set_param('MaskSelfModifiable','on');
        obj.updateFlag = true;
        set_param(obj.componentBlockHandle,'linkstatus','none');
		obj.setBlock()
		obj.setMask()
    end
    
    function set.sides(obj, value)
        if (length(nonzeros(value))~=2)
            error("Electrode must have 2 non-zero dimensions.")
        end
        obj.sides = value;            
        obj.updateBlock;
    end
    
    function set.pos0(obj, value, varargin)
        obj.pos0 = value;
        obj.updateBlock;
    end

    function set.channels(obj, value, varargin)
        obj.channels = value;
        obj.updateBlock;
    end

    function set.V_DC(obj, value, varargin)
        obj.V_DC = value;
        obj.updateBlock;
    end

    function a = get.area(obj)
        a = prod(obj.sides(obj.sides~=0));
        %obj.sides(1) *obj.sides(2);
    end

    function n = get.normal(obj)
        n = [obj.sides==0];
        n = (obj.pos0.*n);
        n = n./max(abs(n));
    end
	
	function P = get.blockSetParameters(obj)
		pn{1} = 'nChannels';
		pn{2} = 'pos0';
		pn{3} = 'area';
		pn{4} = 'V_DC';
		pv{1} = length(obj.channels);
		pv{2} = obj.pos0;
		pv{3} = obj.area;
		pv{4} = obj.V_DC;
		P = {pn, pv};
	end

	function P = get.blockUpdateParameters(obj)
		pn{1} = 'nChannels';
		pn{2} = 'pos0';
		pn{3} = 'area';
		pn{4} = 'V_DC';
		pv{1} = length(obj.channels);
		pv{2} = obj.pos0;
		pv{3} = obj.area;
		pv{4} = obj.V_DC;
		P = {pn, pv};
	end

	function checkBlock(obj)
		if ~strcmp(get_param(obj.componentBlockHandle,'Parameters'), ...
				'nChannels, pos0, area, V_DC')
			throw(MException('Simulink:WrongSFunctionParms',"Wrong" + ...
				" S-Function parameters."));
		end

		if ~strcmp(get_param(obj.componentBlockHandle,'BlockType'), ...
				'M-S-Function')||...
		   ~strcmp(get_param(obj.componentBlockHandle,'FunctionName'),...
				'electrodesf')
			throw(MException('Simulink:WrongBlockType',"Electrode should be a" + ...
				" level 2 s-function block with 'electrodesf' function."));		
		end
		return
	end

	function M = get.blockMask(obj)
		M = "image('electrode.png');" + newline;
		M = strcat(M,"text(0.5, 0.2,'", strcat('V_DC=',num2str(obj.V_DC)), ...
			"V', 'horizontalAlignment', 'center');" + newline);
		M = strcat(M,"port_label('input', 1, char(948));" + newline);
		M = strcat(M,"port_label('output',1, 'pos');" + newline);
		M = strcat(M,"port_label('output',2, 'A');" + newline);
		M = strcat(M,"port_label('output',3, 'V_DC');" + newline);
		for i=1:length(obj.channels)
			M = strcat(M, "port_label('input',", ...
				num2str(i+1), ", 'V",obj.channels(i),"');" + newline);
			M = strcat(M, "port_label('output',", ...
	           num2str(i+3), ", 'V",obj.channels(i),"');" + newline);
		end
	end
end

end

