classdef ElectrodeHW222 < Component
%ELECTRODE222SF Wrapper class for the electrode hardwired 2x2x2 S-function block 
% This class creates an object that collects all the utilities around and 
% connects to a copy of the electrode s-function block.
% System-in-a-block implementation of 6 pairs of electrodes (2 per
% side, cubic electrode housing for the TM). Rotations are x-roll, y-pitch,
% z-yaw;
% 
%         ^z
%         |
%         |___________    -
%        /|           |   :
%       / | ===  ===  |   :   
%      /  | ===  ===  |   : L 
%     / = | =XC  =XD  |   :   
%    / YA | ===  ===  |   :
%   | =   | ===  ===  |   :
%   |   = |___________|___-______> y
%   |  YB/ ========== / 
%   | = / ===ZB===== /
%   |  /            / 
%   | / =====ZA=== /
%   |/ ========== /
%   / ___________/
%  /
% /x
% 
%ELECTRODE 
%
%%% Properties:
%
% * componentBlockName - same as superclass
% * componentBlockHandle - same as superclass
% * attachedModel - same as superclass
% * updateFlag - same as superclass
% * blockSetParameters - parameters passed to block mask during setup 
% (TM_size, TM_gaps, el_areas, el_pos, TM_shadowPos, eps0);
% * blockUpdateParameters - parameters passed to block mask during update
%  (TM_size, TM_gaps, el_areas, el_pos, TM_shadowPos, eps0);
% 
%%% Methods:
%
% * ElectrodeHW222 - object constructor. Should be added to an acme instance 
% through addElectrodeHW222 call. Example:
% el222 = acmeinstance.addElectrodeHW222(BlockName, TM_size, TM_gaps, ...
% el_areas, el_pos, TM_shadowPos, eps0)
%     Output:
%         * el222 - Electrode object. 
%     Inputs:   
%         * BlockName - char. Name of the block in the Simulink model. Must be a
%         Matlab Level 2 "electrodesf" S-Function.
%         * TM_size - 
%         * TM_gaps - 
%         * el_areas - 
%         * el_pos -
%         * TM_shadowPos - 
%         * eps0 - 
% 
% * checkBlock - checks if the block has the right s-function and mask
% parameters;
%
%%%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 18.01.2024;

properties
	TM_size (1,3) double
	TM_gaps (1,3) double
	el_areas (2,3) double
	el_pos (12,3) double
	TM_shadowPos (12,3) double
	eps0 (1,1) double
end

properties (Dependent)
    %area   (1,1) double  %
    %normal (1,3) double  % unitary vector to get the force direction
	blockSetParameters
	blockUpdateParameters
	blockMask
end

methods
    function obj = ElectrodeHW222(varargin)        
        p = inputParser;
        p.addParameter('BlockName', '', @(x) ischar(x));
        p.addParameter('Model', '', @(x) ischar(x));
        p.addParameter('TM_size', [0 0 0]);
		p.addParameter('TM_gaps', [0 0 0]);
		p.addParameter('el_areas', [0 0 0; 0 0 0]);
		p.addParameter('el_pos', zeros(12,3));
		p.addParameter('TM_shadowPos', zeros(12,3));
		p.addParameter('eps0', 8.9e-12);
        p.parse(varargin{:})
        
        blockName = p.Results.BlockName;
        model     = p.Results.Model;
        TM_size   = p.Results.TM_size;
		TM_gaps   = p.Results.TM_gaps;
		el_areas  = p.Results.el_areas;
		el_pos    = p.Results.el_pos;
		TM_shadowPos = p.Results.TM_shadowPos;
		eps0      = p.Results.eps0;
        
        obj.componentBlockName = blockName;
        obj.attachedModel = model;
        obj.componentBlockHandle = obj.getHandle;

		obj.checkBlock;
        obj.TM_size = TM_size;
		obj.TM_gaps = TM_gaps;
		obj.el_areas = el_areas;
		obj.el_pos = el_pos;
		obj.TM_shadowPos = TM_shadowPos;
		obj.eps0 = eps0;
		obj.set_param('MaskSelfModifiable','on');
        obj.updateFlag = true;
        set_param(obj.componentBlockHandle,'linkstatus','none');
		obj.setBlock()
		obj.setMask()
    end
    
    function set.TM_size(obj, value, varargin)
        obj.TM_size = value;
        obj.updateBlock;
    end

	function set.TM_gaps(obj, value, varargin)
        obj.TM_gaps = value;
        obj.updateBlock;
    end

	function set.el_areas(obj, value, varargin)
        obj.el_areas = value;
        obj.updateBlock;
    end

	function set.el_pos(obj, value, varargin)
        obj.el_pos = value;
        obj.updateBlock;
    end

	function set.TM_shadowPos(obj, value, varargin)
        obj.TM_shadowPos = value;
        obj.updateBlock;
    end
	
	function set.eps0(obj, value, varargin)
        obj.eps0 = value;
        obj.updateBlock;
    end
	
	function P = get.blockSetParameters(obj)
		pn{1} = 'TM_size';
		pn{2} = 'TM_gaps';
		pn{3} = 'el_areas';
		pn{4} = 'el_pos';
		pn{5} = 'TM_shadowPos';
		pn{6} = 'eps0';
		pv{1} = obj.TM_size;
		pv{2} = obj.TM_gaps;
		pv{3} = obj.el_areas;
		pv{4} = obj.el_pos;
		pv{5} = obj.TM_shadowPos;
		pv{6} = obj.eps0;
		P = {pn, pv};
	end

	function P = get.blockUpdateParameters(obj)
		pn{1} = 'TM_size';
		pn{2} = 'TM_gaps';
		pn{3} = 'el_areas';
		pn{4} = 'el_pos';
		pn{5} = 'TM_shadowPos';
		pn{6} = 'eps0';
		pv{1} = obj.TM_size;
		pv{2} = obj.TM_gaps;
		pv{3} = obj.el_areas;
		pv{4} = obj.el_pos;
		pv{5} = obj.TM_shadowPos;
		pv{6} = obj.eps0;
		P = {pn, pv};
	end

	function checkBlock(obj)
		if ~strcmp(get_param(obj.componentBlockHandle,'Parameters'), ...
				'TM_size, TM_gaps, el_areas, el_pos, TM_shadowPos, eps0')
			throw(MException('Simulink:WrongSFunctionParms',"Wrong" + ...
				" S-Function parameters."));
		end

		if ~strcmp(get_param(obj.componentBlockHandle,'BlockType'), ...
				'M-S-Function')||...
		   ~strcmp(get_param(obj.componentBlockHandle,'FunctionName'),...
				'electrodes222sf')
			throw(MException('Simulink:WrongBlockType',"Electrode should be a" + ...
				" level 2 s-function block with 'electrodes222sf' function."));		
		end
		return
	end

	function M = get.blockMask(obj)
		M = "image('el222.png');" + newline;
		M = strcat(M,"port_label('input', 1, 'TM pos');" + newline);
		M = strcat(M,"port_label('input', 2, 'TM rot');" + newline);
		M = strcat(M,"port_label('input', 3, 'Vs');" + newline);
		M = strcat(M,"port_label('input', 4, 'V DC');" + newline);
		M = strcat(M,"port_label('input', 5, 'V TM');" + newline);
		M = strcat(M,"port_label('output',1, 'F');" + newline);
		M = strcat(M,"port_label('output',2, 'N');" + newline);
		M = strcat(M,"port_label('output',3, 'C');" + newline);
	end
end

end

