classdef TestMass < Component
%TESTMASS Wrapper class for the a dynamics block that contains TM dynamics.
%   This class creates an object that collects all the utilities around and 
% connects to a copy of a block such as Aerospace blockset's 6DoFs double
% integrators, XHPS test mass dynamics (planned),
% and Simulink standard double integrator (update required).  
% The struct properties are syntactic sugar only. Should be vectorized
% before interfacing with Simulink.
%
%%% Properties:
%
% * componentBlockName   - name of the block in the model;
% * componentBlockHandle - handle of the block in the model;
% * attachedModel - name of the model;
% * updateFlag    - boolean to stop updating block before object is ready;
% * blockSetParameters    - cell 'dict' of parameters k/v to set up block;
% * blockUpdateParameters - cell 'dict' of parameters k/v to update block;
% * mass - mass of the test mass, kg
% * l - size of TM, struct with the 3 dimensions, m
% * gap - gaps between TM and housing, struct with the 3 dimensions, m
% * area - areas of faces,  m2
% * q0 - initial position of TM, m
% * dq0 - initial velocity of TM, m/s
% * volume - volume of the TM, m3
% * density - density of material, kg/m3
% * sides - same as l
% * gaps - same as gap
% * areas - same as aewa
% * vertices - positions of vertices
% * momentOfInertia - moment of inertia of cuboid
% * I - same as momentOfInertia
% * voltage - voltage
% * blockSetParameters - parameters passed to block mask during setup 
% * blockUpdateParameters - parameters passed to block mask during update
%
%%% Methods
% * TestMass - object constructor. Should be added to an acme instance through
% addTestMass call. Can be defined either by mass or density. Example:
% TM = acmeInstance.addTestMass(BlockName, side, gap, 'mass', mass);
% TM = acmeInstance.addTestMass(BlockName, side, gap, 'density', density);
%        
% * Inputs:
%     * BlockName - char. Name of the block in the Simulink model. Must be a
%         Matlab 6DoF Euler Angles Block.
%     * side - m, side of the cube, 1x1 double OR
%               sides of the cuboid, m, 1x3 double
%     * gap - m, size of the gaps (equal sizes), 1x1 double OR
%           sizes of gaps, 1x3 double
%     * mass - kg, mass of the test mass
%     * density - kg/m3, density of the material
%     * voltage - TM polarization voltage, V
% * Outputs:
%     * TM - testmass object
%%%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 21.06.2023;

properties
    mass    (1,1) double
    l    = struct('x', 0, 'y', 0, 'z', 0); 
    gap  = struct('x', 0, 'y', 0, 'z', 0); 
    area = struct('x', 0, 'y', 0, 'z', 0); 
    q0  (1,3) double = [0 0 0]
    dq0 (1,3) double = [0 0 0]
%	voltage (1,1) double = 0;
end

properties (Dependent)
    volume  
    density 
    sides %todo: review if required   
    gaps  %todo: review if required   
    areas %todo: review if required   
    vertices %todo: review if required
    momentOfInertia  
    I
	blockSetParameters
	blockUpdateParameters
	blockMask
end

methods
    function obj = TestMass(varargin)
        % TESTMASS Construct an instance of this class
        % Either mass or density must be defined
        % Syntax 
        % TM = TestMass(sideLength, gap, ...
        %               'mass'/'density', mass/density,...
        %               [geometricFactor]
        % Inputs:
        %    sideLength - m, side of the cube, 1x1 double OR
        %               - m, sides of the cuboid, m, 1x3 double
        %    gap  - m, size of the gaps (equal sizes), 1x1 double OR
        %         - m, sizes of gaps, 1x3 double
        %    mass - kg, mass of the test mass
        %    density - kg/m3, density of the material
        %    voltage - V
                    
        p = inputParser;
        p.addRequired('SideLength');
        p.addRequired('Gap');
%		p.addRequired('voltage');
        p.addParameter('Mass',    0, @(x) isscalar(x));
        p.addParameter('Density', 0, @(x) isscalar(x));
        p.addParameter('q0',      0, @(x) isnumeric(x));
        p.addParameter('dq0',     0, @(x) isnumeric(x));
        
        p.addParameter('Model', '',  @(x) ischar(x));
        p.addParameter('BlockName', '', @(x) ischar(x));
        %p.addParameter('BlocksNames',   [''], @(x) (x));
        %p.addParameter('Solver',''); %todo: review if still required
        
        p.parse(varargin{:});
       
        massTemp = p.Results.Mass;
        densityTemp = p.Results.Density;
        sideLengthTemp = p.Results.SideLength;
        gapsTemp = p.Results.Gap;
        
        if ~xor(massTemp,densityTemp)
            error('either mass or density has to be defined')
        end

        obj.l  = TestMass.initSides(sideLengthTemp);
        obj.gap = TestMass.initGaps(gapsTemp);
        if massTemp == 0
            obj.mass = obj.volume*densityTemp;
        elseif densityTemp == 0
            obj.mass = massTemp;
        end
        %todo: implement Moment of inertia calculation
        obj.q0  = p.Results.q0;
        obj.dq0 = p.Results.dq0;  %todo: review if required, perhaps just a
		%obj.voltage = p.Results.voltage;
		% very unnusual test case?

        obj.componentBlockName = p.Results.BlockName;
        obj.attachedModel = p.Results.Model;
        obj.componentBlockHandle = obj.getHandle;
        obj.updateFlag = true;
        obj.updateBlock;
    end
     
    function obj = attachBlock(obj, model, block)
        obj.attachedModel = model;
        obj.attachedDoubleIntegratorBlock = block;
    end

    function set.mass(obj, value)
        obj.mass = value;
        updateBlock(obj);
    end

    function set.gap(obj, value)
        gs  = TestMass.initGaps(value);
        obj.gap.x = gs(1);
        obj.gap.y = gs(2);
        obj.gap.z = gs(3);
        updateBlock(obj);
    end

    function set.l(obj, value)
         ls  = TestMass.initSides(value);
         obj.l.x = ls(1);
         obj.l.y = ls(2);
         obj.l.z = ls(3);

         updateBlock(obj);
    end

    function set.density(obj, value)
        obj.mass = value*obj.volume;
        updateBlock(obj);
    end

    function set.q0(obj, value)
        obj.q0 = value;
        updateBlock(obj);
    end

    function set.dq0(obj, value)
        obj.dq0 = value;
        updateBlock(obj);
    end

    function v = get.volume(obj)
          x = obj.l.x;
          y = obj.l.y;
          z = obj.l.z;
          v = x*y*z;
    end

    function d = get.density(obj)
        d = obj.mass/obj.volume;
    end

    function s = get.sides(obj)
        s = [obj.l.x obj.l.y obj.l.z];
    end

    function g = get.gaps(obj)
        g = [obj.gap.x obj.gap.y obj.gap.z];
    end

    function a = get.area(obj)
        a.x = obj.l.y * obj.l.z;
        a.y = obj.l.x * obj.l.z;
        a.z = obj.l.x * obj.l.y;
    end

    function a = get.areas(obj)
        a = [obj.area.x obj.area.y obj.area.z];
    end

    function v = get.vertices(obj)
        xv = obj.l.x/2;
        yv = obj.l.y/2;
        zv = obj.l.z/2;
        v1 = [+xv,+yv,+zv];
        v2 = [+xv,+yv,-zv];
        v3 = [+xv,-yv,+zv];
        v4 = [+xv,-yv,-zv];
        v5 = [-xv,+yv,+zv];
        v6 = [-xv,+yv,-zv];
        v7 = [-xv,-yv,+zv];
        v8 = [-xv,-yv,-zv];
        v = [v1, v2, v3, v4, v5, v6, v7, v8];
    end

    function I = get.momentOfInertia(obj)
        I = (obj.mass/12)*[obj.l.x^2,         0,         0;...
                                   0, obj.l.y^2,         0;...
                                   0,         0, obj.l.z^2];
    end

    function I = get.I(obj)
        I = obj.momentOfInertia;
    end

	function P = get.blockSetParameters(obj)
		pn{1} = 'xme_0';
		pn{2} = 'vm_0';
		pn{3} = 'mass_e';
		pn{4} = 'mass_0';
		pn{5} = 'mass_f';
		pn{6} = 'inertia';
		pn{7} = 'inertia_f';
		pn{8} = 'inertia_e';
%		pn{9} = 'voltage';
		pv{1} = obj.q0;
		pv{2} = obj.dq0;
		pv{3} = obj.mass;
		pv{4} = obj.mass;
		pv{5} = obj.mass;
		pv{6} = obj.I;
		pv{7} = obj.I;
		pv{8} = obj.I;
%		pn{9} = obj.voltage;
		P = {pn, pv};
	end

	function P = get.blockUpdateParameters(obj)
		pn{1} = 'xme_0';
		pn{2} = 'vm_0';
		pn{3} = 'mass_e';
		pn{4} = 'mass_0';
		pn{5} = 'mass_f';
		pn{6} = 'inertia';
		pn{7} = 'inertia_f';
		pn{8} = 'inertia_e';
%		pn{9} = 'voltage';
		pv{1} = obj.q0;
		pv{2} = obj.dq0;
		pv{3} = obj.mass;
		pv{4} = obj.mass;
		pv{5} = obj.mass;
		pv{6} = obj.I;
		pv{7} = obj.I;
		pv{8} = obj.I;
%		pv{9} = obj.voltage;
		P = {pn, pv};
	end

	function checkBlock(obj)
		return
	end
	
	function M = get.blockMask(obj)
		M ='';
	end
end

methods (Static)
 function l = initSides(sideLengthTemp)
        if length(sideLengthTemp) == 3
            l = [sideLengthTemp(1),...
                 sideLengthTemp(2),...
                 sideLengthTemp(3)];
        else
            l = [sideLengthTemp, sideLengthTemp, sideLengthTemp];
        end
 end

 function g = initGaps(gapTemp)
        if length(gapTemp) == 3
            g = [gapTemp(1),...
                 gapTemp(2),... 
                 gapTemp(3)];
        else
            g = [gapTemp, gapTemp, gapTemp];
        end
    end

    function area = initAreas(l)
        area.x = l.y * l.z;
        area.y = l.x * l.z;
        area.z = l.x * l.y;
    end    
    
  
end
end

