function diffCaps(block)
%ELECTRODES System-in-a-block implementation of 6 pairs of electrodes (2 per
% side, cubic electrode housing for the TM). Rotations are x-roll, y-pitch,
% z-yaw;
% 
%         ^z
%         |
%         |___________    -
%        /|           |   :
%       / | ===  ===  |   :   
%      /  | ===  ===  |   : L 
%     / = | =XC  =XD  |   :   
%    / YA | ===  ===  |   :
%   | =   | ===  ===  |   :
%   |   = |___________|___-______> y
%   |  YB/ ========== / 
%   | = / ===ZB===== /
%   |  /            / 
%   | / =====ZA=== /
%   |/ ========== /
%   / ___________/
%  /
% /x
% 
% Parameters:
%     TM_size - dimentions of the TM, m, [1x3] double, (Lx, Ly, Lz);
%     TM_gaps - dimentions of the TM-EL housing gaps, m, [1x3] double;
%     el_areas - area of each electrode,  m^2, [2x3] double;
%     el_pos - pos of each electrode i.r.t nominal TM origin, m, [12x3] double
%     TM_shadowsPos - pos of each electrode's shadow i.r.t nominal TM origin, 
%  m, [12x3] double;
%     eps0 - dielectric constant
%
% Block Inputs:
%     TM_pos - instantaneous position of TM, m, [1x3] double;
%     TM_rot - instantaneous orientation of TM, rad,[1x3] double;
%     Vs  - control signals amplitude, [1x12] double;
%     V_DCs - dc biases, [1x12] double;
%     V_TM - TM voltage, [1x1] double
%
% Block Outputs:
%     F - force, N, [1x3] double;
%     N - torque, N.m, [1x3] double;    
%     C - instantaneous capacitance, F, [1x12] double;    
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 18.01.2024;
setup(block);


function setup(block)
block.NumInputPorts  = 12;
block.NumOutputPorts = 6;

block.SetPreCompInpPortInfoToDynamic;
block.SetPreCompOutPortInfoToDynamic;
block.NumDialogPrms     = 0;

for i = 1:12
	block.InputPort(i).Dimensions  =  1;
	block.InputPort(i).DatatypeID  =  0;  
	block.InputPort(i).Complexity  = 'Real';
	block.InputPort(i).DirectFeedthrough = true;
end

for i = 1:6
	block.OutputPort(i).Dimensions  = 1;
	block.OutputPort(i).DatatypeID  = 0;  % double
	block.OutputPort(i).Complexity  = 'Real';
	block.OutputPort(i).SamplingMode = 'Sample';
end

block.SampleTimes = [0 0];

block.SimStateCompliance = 'DefaultSimState';

block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
block.RegBlockMethod('Start', @Start);
block.RegBlockMethod('Outputs', @Outputs);     % Required
block.RegBlockMethod('Terminate', @Terminate); % Required
block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);

function DoPostPropSetup(block)
   block.NumDworks = 0;

function Start(~)
    
 
function Outputs(block)

Cxa = block.InputPort(1).Data; % XA: InputPort(1)
Cxb = block.InputPort(2).Data; % XB: InputPort(2)
Cxc = block.InputPort(3).Data; % XC: InputPort(3)
Cxd = block.InputPort(4).Data; % XD: InputPort(4)
Cya = block.InputPort(5).Data; % YA: InputPort(5)
Cyb = block.InputPort(6).Data; % YB: InputPort(6)
Cyc = block.InputPort(7).Data; % YC: InputPort(7)
Cyd = block.InputPort(8).Data; % YD: InputPort(8)
Cza = block.InputPort(9).Data; % ZA: InputPort(9)
Czb = block.InputPort(10).Data;% ZB: InputPort(10)
Czc = block.InputPort(11).Data;% ZC: InputPort(11)
Czd = block.InputPort(12).Data;% ZD: InputPort(12)

%deltaC_x = Cxa + Cxb - Cxc - Cxd
%deltaC_y = Cya + Cyb - Cyc - Cyd 
%deltaC_z = Cza + Czb - Czc - Czd
%deltaC_roll  = 1/(1/Cya + 1/Cyd) - 1/(1/Cyb + 1/Cyc)
%deltaC_yaw   = 1/(1/Cxa + 1/Cxd) - 1/(1/Cxb + 1/Cxc)
%deltaC_pitch = 1/(1/Cza + 1/Czd) - 1/(1/Czb + 1/Czc)

block.OutputPort(1).Data = Cxa + Cxb - Cxc - Cxd; %deltaC_x
block.OutputPort(2).Data = Cya + Cyb - Cyc - Cyd; %deltaC_y        
block.OutputPort(3).Data = Cza + Czb - Czc - Czd; %deltaC_z        
block.OutputPort(4).Data = 1/(1/Cya + 1/Cyd) - 1/(1/Cyb + 1/Cyc);
block.OutputPort(6).Data = 1/(1/Cxa + 1/Cxd) - 1/(1/Cxb + 1/Cxc);
block.OutputPort(5).Data = 1/(1/Cza + 1/Czd) - 1/(1/Czb + 1/Czc);

%function Update(block)

    %block.Dwork(1).Data = block.InputPort(1).Data;

%function Derivatives(block)

function Terminate(block)

function SetInpPortFrameData(block, idx, fd)

  block.InputPort(idx).SamplingMode = fd;
  for i = 1:block.NumOutputPorts
    block.OutputPort(i).SamplingMode  = fd;
  end