function electrodes222sf(block)
%ELECTRODE222SF System-in-a-block implementation of 6 pairs of electrodes (2 per
% side, cubic electrode housing for the TM). Rotations are x-roll, y-pitch,
% z-yaw;
% 
%         ^z
%         |
%         |___________    -
%        /|           |   :
%       / | ===  ===  |   :   
%      /  | ===  ===  |   : L 
%     / = | =XC  =XD  |   :   
%    / YA | ===  ===  |   :
%   | =   | ===  ===  |   :
%   |   = |___________|___-______> y
%   |  YB/ ========== / 
%   | = / ===ZB===== /
%   |  /            / 
%   | / =====ZA=== /
%   |/ ========== /
%   / ___________/
%  /
% /x
% 
% Parameters:
%     TM_size - dimentions of the TM, m, [1x3] double, (Lx, Ly, Lz);
%     TM_gaps - dimentions of the TM-EL housing gaps, m, [1x3] double;
%     el_areas - area of each electrode,  m^2, [2x3] double;
%     el_pos - pos of each electrode i.r.t nominal TM origin, m, [12x3] double
%     TM_shadowsPos - pos of each electrode's shadow i.r.t nominal TM origin, 
%  m, [12x3] double;
%     eps0 - dielectric constant
%
% Block Inputs:
%     TM_pos - instantaneous position of TM, m, [1x3] double;
%     TM_rot - instantaneous orientation of TM, rad,[1x3] double;
%     Vs  - control signals amplitude, [1x12] double;
%     V_DCs - dc biases, [1x12] double;
%     V_TM - TM voltage, [1x1] double
%
% Block Outputs:
%     F - force, N, [1x3] double;
%     N - torque, N.m, [1x3] double;    
%     C - instantaneous capacitance, F, [1x12] double;    
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 18.01.2024;

setup(block);

function setup(block)
block.SetPreCompInpPortInfoToDynamic;
block.SetPreCompOutPortInfoToDynamic;
block.NumDialogPrms     = 6;
% TM_size, TM_gaps, el_areas, el_pos, TM_shadowPos, eps0

% INPUT PORTS
block.NumInputPorts  = 5;

% TM position, (x,y,z), [1x3] double
block.InputPort(1).Dimensions  =  3;
block.InputPort(1).DatatypeID  =  0;  
block.InputPort(1).Complexity  = 'Real';
block.InputPort(1).DirectFeedthrough = true;

% TM rotation, (roll,pitch,yaw), [1x3] double
block.InputPort(2).Dimensions  =  3;
block.InputPort(2).DatatypeID  =  0;  
block.InputPort(2).Complexity  = 'Real';
block.InputPort(2).DirectFeedthrough = true;

% Vs Control voltages, (V_XAx, V_XBx, ...., V_ZDz,...
%                       V_XAyaw, V_XByaw, ...., V_ZDpitch,...)
block.InputPort(3).Dimensions  =  24;
block.InputPort(3).DatatypeID  =  0;  
block.InputPort(3).Complexity  = 'Real';
block.InputPort(3).DirectFeedthrough = true;

% V_DCs, (V_XA_DC, V_XB_DC, ...., V_ZD_DC)
block.InputPort(4).Dimensions  =  12;
block.InputPort(4).DatatypeID  =  0;  
block.InputPort(4).Complexity  = 'Real';
block.InputPort(4).DirectFeedthrough = true;

% V_TM
block.InputPort(5).Dimensions  =  1;
block.InputPort(5).DatatypeID  =  0;  
block.InputPort(5).Complexity  = 'Real';
block.InputPort(5).DirectFeedthrough = true;

% OUTPUT PORTS
%     NumOutputPorts = C + gap + pos + nChannels;
block.NumOutputPorts = 3;
% F, (Fx,Fy,Fz), [1x3] double
block.OutputPort(1).Dimensions  = 3;
block.OutputPort(1).DatatypeID  = 0;  % double
block.OutputPort(1).Complexity  = 'Real';
block.OutputPort(1).SamplingMode = 'Sample';
% N, (Nroll,Npitch,Nyaw), [1x3] double
block.OutputPort(2).Dimensions  = 3;
block.OutputPort(2).DatatypeID  = 0;  % double
block.OutputPort(2).Complexity  = 'Real';
block.OutputPort(2).SamplingMode = 'Sample';
% C, (C_XA, C_XB, ..., C_ZD)
block.OutputPort(3).Dimensions  = 12;
block.OutputPort(3).DatatypeID  = 0;  % double
block.OutputPort(3).Complexity  = 'Real';
block.OutputPort(3).SamplingMode = 'Sample';

block.SampleTimes = [0 0];

block.SimStateCompliance = 'DefaultSimState';

block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
block.RegBlockMethod('Start', @Start);
block.RegBlockMethod('Outputs', @Outputs);     % Required
block.RegBlockMethod('Terminate', @Terminate); % Required
block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);

%end setup

function DoPostPropSetup(block)
   block.NumDworks = 0;

function Start(~)
     
function Outputs(block)
% TM_size, TM_gaps, el_areas, el_pos, TM_shadowPos, eps0
	TM_size  = block.DialogPrm(1).Data;
	TM_gaps  = block.DialogPrm(2).Data;
	el_areas = block.DialogPrm(3).Data;
	el_pos   = block.DialogPrm(4).Data;
	TM_shadowPos = block.DialogPrm(5).Data;
	eps0     = block.DialogPrm(6).Data;

	TM_pos = block.InputPort(1).Data;
	TM_rot = block.InputPort(2).Data;
    Vs     = block.InputPort(3).Data; 
    V_DCs  = block.InputPort(4).Data;
    V_TM   = block.InputPort(5).Data;

	% pitch roll yaw
	pitch = TM_rot(1);
	roll  = TM_rot(2);
	yaw   = TM_rot(3);
	dcm = angle2dcm(yaw, pitch, roll);
	% assume small displacement and small rotation
    
	shadownewpos = (TM_shadowPos + repmat(TM_pos,1,12)');
	shadowrotated = zeros(size(shadownewpos));
	for i = 1:12
		shadowrotated(i,:) = dcm*(shadownewpos(i,:)');
	end

    TM_el_gaps = el_pos - shadowrotated;
	d = sqrt(sum(TM_el_gaps(:,:).^2,2));
	
	A = zeros(12,1);
	A(1) = el_areas(1,1);
	A(2) = el_areas(2,1);
	A(3) = el_areas(2,1);
	A(4) = el_areas(1,1);
	A(5) = el_areas(1,2);
	A(6) = el_areas(2,2);
	A(7) = el_areas(2,2);
	A(8) = el_areas(1,2);
	A(9) = el_areas(1,3);
	A(10)= el_areas(2,3);
	A(11)= el_areas(2,3);
	A(12)= el_areas(1,3);

	C = eps0 * A./d;
    % f = (VDC + VAC)^2 * eps0*A/2/d^2
	F_XAx     = (V_DCs(1) + Vs(1)  - V_TM)^2*eps0*A(1)/2/d(1)^2;
	F_XBx     = (V_DCs(2) + Vs(2)  - V_TM)^2*eps0*A(2)/2/d(2)^2;
	F_XCx     = (V_DCs(3) + Vs(3)  - V_TM)^2*eps0*A(3)/2/d(3)^2;
	F_XDx     = (V_DCs(4) + Vs(4)  - V_TM)^2*eps0*A(4)/2/d(4)^2;
	F_XAyaw   = (V_DCs(1) + Vs(13) - V_TM)^2*eps0*A(1)/2/d(1)^2;
	F_XByaw   = (V_DCs(2) + Vs(14) - V_TM)^2*eps0*A(2)/2/d(2)^2;
	F_XCyaw   = (V_DCs(3) + Vs(15) - V_TM)^2*eps0*A(3)/2/d(3)^2;
	F_XDyaw   = (V_DCs(4) + Vs(16) - V_TM)^2*eps0*A(4)/2/d(4)^2;
	% 
	F_YAy     = (V_DCs(5) + Vs(5)  - V_TM)^2*eps0*A(5)/2/d(5)^2;
	F_YBy     = (V_DCs(6) + Vs(6)  - V_TM)^2*eps0*A(6)/2/d(6)^2;
	F_YCy     = (V_DCs(7) + Vs(7)  - V_TM)^2*eps0*A(7)/2/d(7)^2;
	F_YDy     = (V_DCs(8) + Vs(8)  - V_TM)^2*eps0*A(8)/2/d(8)^2;
	F_YAroll  = (V_DCs(5) + Vs(17) - V_TM)^2*eps0*A(5)/2/d(5)^2;
	F_YBroll  = (V_DCs(6) + Vs(18) - V_TM)^2*eps0*A(6)/2/d(6)^2;
	F_YCroll  = (V_DCs(7) + Vs(19) - V_TM)^2*eps0*A(7)/2/d(7)^2;
	F_YDroll  = (V_DCs(8) + Vs(20) - V_TM)^2*eps0*A(8)/2/d(8)^2;
	
	F_ZAz     = (V_DCs( 9)+ Vs( 9) - V_TM)^2*eps0*A( 9)/2/d( 9)^2;
	F_ZBz     = (V_DCs(10)+ Vs(10) - V_TM)^2*eps0*A(10)/2/d(10)^2;
	F_ZCz     = (V_DCs(11)+ Vs(11) - V_TM)^2*eps0*A(11)/2/d(11)^2;
	F_ZDz     = (V_DCs(12)+ Vs(12) - V_TM)^2*eps0*A(12)/2/d(12)^2;
	F_ZApitch = (V_DCs( 9)+ Vs(21) - V_TM)^2*eps0*A( 9)/2/d( 9)^2;
	F_ZBpitch = (V_DCs(10)+ Vs(22) - V_TM)^2*eps0*A(10)/2/d(10)^2;
	F_ZCpitch = (V_DCs(11)+ Vs(23) - V_TM)^2*eps0*A(11)/2/d(11)^2;
	F_ZDpitch = (V_DCs(12)+ Vs(24) - V_TM)^2*eps0*A(12)/2/d(12)^2;
	

	F = [F_XAx + F_XBx - F_XCx - F_XDx;
	     F_YAy + F_YBy - F_YCy - F_YDy;
	     F_ZAz + F_ZBz - F_ZCz - F_ZDz];  
	%r = zeros(1,12);
	%torque arms hardwired
	r_XA = abs(el_pos(1,2)); % XA
	r_XB = abs(el_pos(2,2)); % XB
	r_XC = abs(el_pos(3,2)); % XC
	r_XD = abs(el_pos(4,2)); % XD
	r_YA = abs(el_pos(5,3)); % YA
	r_YB = abs(el_pos(6,3)); % YB
	r_YC = abs(el_pos(7,3)); % YC
	r_YD = abs(el_pos(8,3)); % YD
	r_ZA = abs(el_pos(9,1)); % ZA
	r_ZB = abs(el_pos(10,1));% ZB
	r_ZC = abs(el_pos(11,1));% ZC
	r_ZD = abs(el_pos(12,1));% ZD
	
    
	N = [F_YBroll *r_YB + F_YCroll *r_YC - F_YAroll *r_YA - F_YDroll *r_YD;
	     F_ZBpitch*r_ZB + F_ZCpitch*r_ZC - F_ZApitch*r_ZA - F_ZDpitch*r_ZD;
	     F_XByaw  *r_XB + F_XCyaw  *r_XC - F_XAyaw  *r_XA - F_XDyaw  *r_XD];  

    block.OutputPort(1).Data = F;
    block.OutputPort(2).Data = N;
	block.OutputPort(3).Data = C;

       
%function Update(block)

    %block.Dwork(1).Data = block.InputPort(1).Data;

%function Derivatives(block)

function Terminate(block)

function SetInpPortFrameData(block, idx, fd)

  block.InputPort(idx).SamplingMode = fd;
  for i = 1:block.NumOutputPorts
    block.OutputPort(i).SamplingMode  = fd;
  end