function electrodesf(block)
%ELECTRODESF 
% MSFUNTMPL_BASIC A Template for a Level-2 MATLAB S-Function
%
%%% Block Parameters:
% * nChannels - number of channels of the electrode 
% * pos0 - nominal position of electrode, irt. to TM CoM, m
% * area - effective area of electrode, m2
% * V_DC - DC bias added to all channels, V
%  
%%% Block Inputs:
% * delta - displacement of electrode, m
% * V_i - voltage of channel i, V
%
%%% Block Outputs:
% * pos - initial position + displacement, m
% * A - effective area of electrode, m2
% * V_DC - DC bias, V
% * V_i - voltage of channel i, V
%
%%%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 18.08.2023;

setup(block);

%endfunction

function setup(block)
block.SetPreCompInpPortInfoToDynamic;
block.SetPreCompOutPortInfoToDynamic;
block.NumDialogPrms     = 4;

nChannels = uint32(block.DialogPrm(1).Data);

% INPUT PORTS
block.NumInputPorts  = 1 + nChannels;
% Displacement
block.InputPort(1).Dimensions        = 3;
block.InputPort(1).DatatypeID  = 0;  % double
block.InputPort(1).Complexity  = 'Real';
block.InputPort(1).DirectFeedthrough = false;

% Control inputs
for i=1:nChannels
    block.InputPort(i+1).Dimensions        = 1;
    block.InputPort(i+1).DatatypeID  = 0;  % double
    block.InputPort(i+1).Complexity  = 'Real';
    block.InputPort(i+1).DirectFeedthrough = true;
end

% OUTPUT PORTS
%     NumOutputPorts = position + area + Vdc_bias + nChannels;
block.NumOutputPorts = 3 + nChannels;
% Position
block.OutputPort(1).Dimensions  = 3;
block.OutputPort(1).DatatypeID  = 0;  % double
block.OutputPort(1).Complexity  = 'Real';
block.OutputPort(1).SamplingMode = 'Sample';

%Area
block.OutputPort(2).Dimensions  = 1;
block.OutputPort(2).DatatypeID  = 0;  % double
block.OutputPort(2).Complexity  = 'Real';
block.OutputPort(2).SamplingMode = 'Sample';

%V_DC
block.OutputPort(3).Dimensions  = 1;
block.OutputPort(3).DatatypeID  = 0;  % double
block.OutputPort(3).Complexity  = 'Real';
block.OutputPort(3).SamplingMode = 'Sample';

%V_AC
for i=1:nChannels
    block.OutputPort(3+i).Dimensions  = 1;
    block.OutputPort(3+i).DatatypeID  = 0;  % double
    block.OutputPort(3+i).Complexity  = 'Real';
end
% Register parameters

% Register sample times
%  [0 offset]            : Continuous sample time
%  [positive_num offset] : Discrete sample time
%
%  [-1, 0]               : Inherited sample time
%  [-2, 0]               : Variable sample time
block.SampleTimes = [0 0];

block.SimStateCompliance = 'DefaultSimState';

block.RegBlockMethod('PostPropagationSetup',    @DoPostPropSetup);
%block.RegBlockMethod('InitializeConditions', @InitializeConditions);
block.RegBlockMethod('Start', @Start);
block.RegBlockMethod('Outputs', @Outputs);     % Required
%block.RegBlockMethod('Update', @Update);
%block.RegBlockMethod('Derivatives', @Derivatives);
block.RegBlockMethod('Terminate', @Terminate); % Required

block.RegBlockMethod('SetInputPortSamplingMode', @SetInpPortFrameData);

%end setup

function DoPostPropSetup(block)
  block.NumDworks = 4;
  
  block.Dwork(1).Name            = 'nChannels';
  block.Dwork(1).Dimensions      = 1;
  block.Dwork(1).DatatypeID      = 7;      % int
  block.Dwork(1).Complexity      = 'Real'; % real
  block.Dwork(1).UsedAsDiscState = false;
  
  block.Dwork(2).Name            = 'pos0';
  block.Dwork(2).Dimensions      = 3;
  block.Dwork(2).DatatypeID      = 0;      % double
  block.Dwork(2).Complexity      = 'Real'; % real
  block.Dwork(2).UsedAsDiscState = false;

  block.Dwork(3).Name            = 'area';
  block.Dwork(3).Dimensions      = 1;
  block.Dwork(3).DatatypeID      = 0;      % double
  block.Dwork(3).Complexity      = 'Real'; % real
  block.Dwork(3).UsedAsDiscState = false;

  block.Dwork(4).Name            = 'V_DC';
  block.Dwork(4).Dimensions      = 1;
  block.Dwork(4).DatatypeID      = 0;      % double
  block.Dwork(4).Complexity      = 'Real'; % real
  block.Dwork(4).UsedAsDiscState = false;

  
%function InitializeConditions(block)

%%
function Start(block)
    block.Dwork(1).Data = uint32(block.DialogPrm(1).Data); %nChannels
    block.Dwork(2).Data = block.DialogPrm(2).Data; %pos0
    block.Dwork(3).Data = block.DialogPrm(3).Data; %area
    block.Dwork(4).Data = block.DialogPrm(4).Data; %V_DC


%end Start

%%
function Outputs(block)
    %pos = pos0 + dx
    block.OutputPort(1).Data = block.Dwork(2).Data + block.InputPort(1).Data; 
    block.OutputPort(2).Data = block.Dwork(3).Data; %area
    block.OutputPort(3).Data = block.Dwork(4).Data; %V_DC
    for i=1:block.Dwork(1).Data %nChannel
         block.OutputPort(i+3).Data = block.InputPort(i+1).Data;
    end

%function Update(block)

    %block.Dwork(1).Data = block.InputPort(1).Data;

%function Derivatives(block)

function Terminate(block)

function SetInpPortFrameData(block, idx, fd)
  
  block.InputPort(idx).SamplingMode = fd;
  for i = 1:block.NumOutputPorts
    block.OutputPort(i).SamplingMode  = fd;
  end