classdef Satellite
%SATELLITE Object for quick and dirty orbital dynamic estimations and other
% properties of the spacecraft that might be required for accelerometer
% constraints without needing to invoke the whole XHPS library.
% Default values are for GRACE/GRACE-FO
%
%%% Properties:
% * mass - mass of the spacecraft, kg (470 kg)
% * altitude - altitude over mean sea level, km (450 km)
% * Cd - drag coefficient, unitless (2.25, from F. Wöske)
% * area - cross section area, normal to velocity (0.9 m2)
% * orbitalVelocity - velocity assuming circular orbit;
%
%%% Methods:
% * Satellite - object constructor. Syntax:
% sat = Satellite(mass, altitude, Cd, area)
% 
% * typicalDragAcc - returns a value of drag decceleration assuming a
% Harris-Priester atmospheric model and circular orbit.
%
% * harrispriester - returns atmospheric density for altitude.
%
%%%  Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 21.08.2023;

properties
    mass     (1,1) double = 470;  % kg
    altitude (1,1) double = 450;  % km over MSL
    Cd       (1,1) double = 2.25; % unitless, from F. Wöske
    area     (1,1) double = 0.9;  % m2
end

properties(Dependent)
    orbitalVelocity % assume ideal circular
end

methods
    function obj = Satellite()
           
    end
    
    function v = get.orbitalVelocity(obj)    
        v = sqrt(3.986e14/(obj.altitude+6400)*1e-3);
    end

    function acc = typicalDragAcc(obj)
        rho = Satellite.harrispriester(obj.altitude);
        acc = 0.5*rho*obj.Cd*obj.area*obj.orbitalVelocity^2 /obj.mass;
    end
end    

methods (Static)
    function rho = harrispriester(h)
        v = [100 497400.0 497400.0 
        120 24900.0 24900.0
        130 8377.0 8710.0 
        140 3899.0 4059.0 
        150 2122.0 2215.0 
        160 1263.0 1344.0 
        180 528.3 601.0 
        190 361.7 429.7 
        170 800.8 875.8
        200 255.7 316.2 
        210 183.9 239.6 
        220 134.1 185.3
        230 99.49 145.5 
        240 74.88 115.7 
        250 57.09 93.08 
        260 44.03 75.55 
        270 34.30 61.82 
        280 26.97 50.95 
        290 21.39 42.26 
        300 17.08 35.26
        320 10.99 25.11
        340 7.214 18.19
        360 4.824 13.37 
        380 3.274 9.955
        400 2.249 7.492 
        420 1.558 5.684
        440 1.091 4.355
        460 0.7701 3.362
        480 0.5474 2.612
        500 0.3916 2.042
        520 0.2819 1.605
        540 0.2042 1.267
        560 0.1488 1.005
        580 0.1092 0.7997
        600 0.08070 0.6390
        620 0.06012 0.5123
        640 0.04519 0.4121
        660 0.03430 0.3325
        680 0.02632 0.2691
        700 0.02043 0.2185
        720 0.01607 0.1779
        740 0.01281 0.1452
        760 0.01036 0.1190
        780 0.008496 0.09776
        800 0.007069 0.08059
        840 0.004680 0.05741
        880 0.003200 0.04210
        920 0.002210 0.03130
        960 0.001560 0.02360
        1000 0.001150 0.01810];
        rho = interp1(v(:,1),v(:,3),h)*1e-12; %rho in kg/m³
     end
    end
end

