function varargout = gasBrownianNoise(varargin)
%THERMALNOISE uses the 4 kb Beta model for white, thermal noise
% Considering a body inserted on a infinite large cavity filled with Knudsen
% gas, the ASD of forces acting on the body due to collisions with the gas can
% be calculated with:
%
%    S^1/2 = sqrt(4*kb*T*beta) (white noise)
%
% Also includes a semi-empirical correction for finite cavity.
%
% Syntax:
%   S_1/2 = gasBrownianNoise(vp, T , s, d[, m0])
%     or
%   [Sx_1/2 Sy_1/2 Sz_1/2] = 
%          gasBrownianNoise(vp, T , [sx sy sz], [dx dy dz], m0)
%
% Input: 
%   p  - pressure, Pa
%   T  - temperature, K
%   s  - length of side, m
%     or [sx, sy, sz] -  length of sides, m
%   d  - gap size, m
%     or [dx, dy, dz] -  gap sizes, m
%   m0 - mass of molecule
%
% Output:
%   S^1/2 - thermal noise ASD, constant over freqs, N/sqrtHz
%     or [S_x^1/2, S_y^1/2, S_z^1/2] - thermal noise ASD in the 3 axis
%   
% See also: 
%   Cavalleri, et al. Phys Let A (2010)
%   Cavalleri, et al. PRL (2009)
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 03.Nov.2022;

kb = 1.38e-23; %J K-1
m0_default = 3e-26; %water molecule weight in kg;

par = inputParser;
par.addRequired('p');
par.addRequired('T');
par.addRequired('s');
par.addRequired('d');
par.addOptional('m0',m0_default);
parse(par,varargin{:});
p  = par.Results.p;
T  = par.Results.T;
s  = par.Results.s;
d  = par.Results.d;
m0 = par.Results.m0;

parse(par,varargin{:});

if (length(s) == 1) && (length(d) == 1)
    thermal = sqrt(4*kb*T*gasDampingCoeff(p,T,s,d,m0));
    varargout = {thermal};
else
    ax = s(2)*s(3);
    ay = s(1)*s(3);
    az = s(1)*s(2);
    thermalx = sqrt(4*kb*T*gasDampingCoeff(p,T,s(1),d(1),m0,ax,ay,az));
    thermaly = sqrt(4*kb*T*gasDampingCoeff(p,T,s(2),d(2),m0,ay,ax,az));
    thermalz = sqrt(4*kb*T*gasDampingCoeff(p,T,s(3),d(3),m0,az,ax,ay));
    varargout = {thermalx, thermaly, thermalz};
end
end

