function beta_coeff = gasDampingCoeff(varargin)
%GASDAMPINGCOEFF calculates the coefficient for 4 kb Beta model
%   Considering a body inserted on a infinite large cavity filled with Knudsen
% gas, the ASD of forces acting on the body due to collisions with the gas can
% be calculated with:
%
%    S^1/2 = sqrt(4*kb*T*beta)
%
% Also includes a semi-empirical correction for finite cavity.
%
% Syntax:
%   beta_coeff = gasDampingCoeff(p,T,s,d [, m0, A, A2, A3])
%
% Input: 
%   p  - pressure, Pa
%   T  - temperature, K
%   s  - length of side, m
%   d  - gap size, d
%   m0 - kg, mass of molecule, default water
%   A  - area of face (normal), m2, optional, if not provided will assume A = s2;
%   A2 - area of face (perp), m2, optional, if not provided assume A2 = A3 = A;
%   A3 - area of face (perp)
%
% Output:
%   beta_coeff - gas damping coefficient for that axis
%   
% See also: 
%   Cavalleri, et al. Phys Let A (2010)
%   Cavalleri, et al. PRL (2009)
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Last revision: 03.Nov.2022;


kb = 1.38e-23; %J K-1
m0_default = 3e-26; %water molecule weight in kg;

par = inputParser;
par.addRequired('p');
par.addRequired('T');
par.addRequired('s');
par.addRequired('d');
par.addOptional('m0',m0_default);
par.addOptional('A',-1);
par.addOptional('A2',-1);
par.addOptional('A3',-1);

parse(par,varargin{:});

p  = par.Results.p;
T  = par.Results.T;
s  = par.Results.s;
d  = par.Results.d;
m0 = par.Results.m0;
A  = par.Results.A;
A2 = par.Results.A2;
A3 = par.Results.A3;

if A==-1
    % areas not provided, assume square
    A = s^2;
end


if (A2 == -1) && (A3 == -1)
    % one area provided, assume cube
    A2 = A;
    A3 = A;
end
beta_inf = p*sqrt(2*m0/(pi*kb*T))*((2+pi/2)*A + A2 + A3);
beta_coeff = beta_inf/(log(s/d)*(d/s)^2); % correction for finite chamber
end


