%% Electrostatic ACC Model
% Following the description in Davor Mance's 2012 thesis. References are from
% that, unless otherwise mentioned. The capacitance sensor model 
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% last review: 19.10.2022
clear;

mdl = 'LISA_EA';
load_system(mdl);

%% Simulation parameters
% frequency and time
stopTime = 1e4;
nSteps = 1e4;
tStep = stopTime/nSteps;
fSample = 1/(stopTime/nSteps);
t = (0:tStep:(stopTime-tStep))';
enbw = 1/stopTime;
freq = logspace(-4,0,100);
% Physical constants
eps0 = 8.86e-12; %F/m, vacuum permittivity
kb = 1.3806e-23; %boltzmann
T = 300; % K


%% Accelerometer Parameters (Ch.2)
% TM
l_TM = 0.030; %m,
mass = .54; %kg
gap0 = 0.001; % m 
area = l_TM^2; %m2
p = 1e-5; %Pa
dblint = setDblIntTrans(mdl, 'dblInt', mass, gap0);

% Electrodes
% There are four electrodes, 2 on the faces on the cage, at potential -V and +V,
% and 2 faces of the TM, at same potential
%     setElectrode(model, block, area, pos0, V_DC, [V_AC], [f], [solver])
% Vd = 5; %V detection rms voltage
% Vp = 1; %V polarizing voltage 
% fCar = 1e5;
% Vxbias = 16;
% el1 = setElectrode(mdl,'electrode1',area,-(l_TM/2+gap0),0 ,0 ,fCar); %Cage left
% el2 = setElectrode(mdl,'electrode2',area,-l_TM/2       ,Vp,Vd,fCar); %TM left
% el3 = setElectrode(mdl,'electrode3',area,+l_TM/2       ,Vp,Vd,fCar); %TM right
% el4 = setElectrode(mdl,'electrode4',area,+(l_TM/2+gap0),0 ,0 ,fCar); %Cage right
%   
Vd = 5; %V detection rms voltage "The TM is modulated at f0 = 100 kHz using the
% injection electrodes, which induce an oscillating voltage in the TM, VTM ,
% of about one tenth of the injected voltage" "VTM = 0.5 V"
V_TM = 0.5;
Vp = 0; %V polarizing voltage #TODO
Vx0 = 16; %V rm
fCar = 1e5;
el1 = setElectrode(mdl,'electrode1',area,-(l_TM/2+gap0), 0, 0 ,fCar); %Cage left
el2 = setElectrode(mdl,'electrode2',area,-l_TM/2       , 0, V_TM,fCar); %TM left
el3 = setElectrode(mdl,'electrode3',area,+l_TM/2       , 0, V_TM,fCar); %TM right
el4 = setElectrode(mdl,'electrode4',area,+(l_TM/2+gap0), 0, 0 ,fCar); %Cage right
  
% Plane capacitors, each formed by two electrodes
%      setPlaneCapacitor(model, block, eps0)
cap1 = setPlaneCapacitor(mdl, 'planeCapacitor1',  eps0); 
cap2 = setPlaneCapacitor(mdl, 'planeCapacitor2',  eps0); 
%capSensorGain = 1.8e13;
Cf = 10e-12; %F
capSensorGain = 10*4*V_TM/Cf; %IA, BPF, cap sensor
%capSensorNoiseASD = @(f) 5.0124e-19 + 1.852e-22./f;
%capSensorNoiseASD = @(f) 5.0124e-19*f.^0;
capSensorNoiseASD = @(f) 5.0124e-21*f.^0;
capSensorNoiseTS  = [t timeseriesFromASD(capSensorNoiseASD, enbw, fSample, nSteps).Data];     
% Capacitive sensing
nominal_cap = 1.15e-12;% F 
Cmax = 0.12e-12; %Tab. B-1
fTypical = 1e-2; %Hz            
% Control
Kp = 1;
Ki = 0.0;
Kd = 10;
driverGain =-1;

gasThermalNoiseASD = @(f) gasBrownianNoise(p,T,l_TM,gap0)/mass + 1.6e-15./sqrt(f);
gasThermalNoiseTS = [t timeseriesFromASD(gasThermalNoiseASD, enbw, fSample, nSteps).Data];     
%%
%bbb = [all_non_grav_forces_A_x(1:50000); all_non_grav_forces_A_x(50000:-1:1)];
%aaa = [t bbb];
%% Simulating
mdlOutput = sim(mdl);    

%% Analysing the spectrum

[axx,fxx] = flpsd('ts', mdlOutput.simout.Data, 'fs', fSample, 'Kdes', 100, ...
                  'Lmin', 1, 'Jdes', 1000, 'win', 'hanning', 'olap', 30, ...
                  'order', 0, 'scale', 'ASD', 'LTPDAmode', false);

% Comparing the results
%  Capacitance sensor
io(1) = linio(strcat(mdl,'/Minus1'),1,'input');
io(2) = linio(strcat(mdl,'/simplifiedCapSensor'),1,'output');
readout2capTF = getTFresponse(mdl, io, fxx); % TF between readout and acc signal
% figure(1);
% loglog(fxx,axx./readout2capTF,fxx,capSensorNoiseASD(fxx),'linewidth',2);
% legend('Simulated measurement','Simulated white noise input');
% xlabel('Frequency [Hz]');
% ylabel('Capacitance [F]');
% 
% % Accelerometer readout
io(1) = linio(strcat(mdl,'/Gain'),1,'input');
io(2) = linio(strcat(mdl,'/simplifiedCapSensor'),1,'output');
readout2accTF = getTFresponse(mdl, io, fxx); % TF between readout and acc signal
% figure(2);
% loglog(fxx,axx./readout2accTF);
% legend('Simulated measurement','Simulated cap noise input');
% xlabel('Frequency [Hz]');
% ylabel('Acceleration [m/s2]');

%%
dof = 'x';  
[noises, sys] = nbFromSimulink(mdl, freq, 'dof', dof);  
nb = nbGroupNoises(mdl, noises, sys);  
matlabNoisePlot(nb);  
loglog(fxx, axx./readout2accTF)

