%% ACME - Capacitive Sensor Simulator
% All references are from Mance's 2012 thesis, unless otherwise noted.
% This model is calculated in the frequency range 1e-4 - 1Hz. Effects of the
% 100kHz injection signal are accounted for but the signal is not fully
% simulated.
% Arthur Reis, 29.11.2022
clear;

mdl = 'capSensor_complete';
load_system(mdl);
%% Simulation parameters
% frequency and time
stopTime = 1e4;
nSteps = 1e6;
tStep = stopTime/nSteps;
fSample = 1/(stopTime/nSteps);
t = (0:tStep:(stopTime-tStep))';
enbw = 1/stopTime;
freq = logspace(-4,0,100);
% Physical constants
eps0 = 8.86e-12; %F/m, vacuum permittivity
kb = 1.3806e-23; %boltzmann
T = 300; % K

%% Accelerometer Parameters
% TM, Ch.2 
TM_side = 0.046; %m,
mass = 2; %kg
nominal_gap = 0.004; % m 
area = TM_side^2; %m2

% Capacitive sensing
nominal_cap = 1.15e-12;% F 
Cmax = 0.12e-12; %Tab. B-1
fTypical = 1e-2; %Hz

%% Voltage Reference
Vd = 4.8; % Vpp, Ch. 2.6
fDetection = 1e5; %Hz 
capAttenuation = 0.123; %Eq 2.60, from ratio between injection
% and all other capacitances
VRefUNoiseASD = @(f) (125e-9.*f.^0) + 1.5e-7.*f.^-.5; %V/sqrt(Hz)
%Ch. 2.5, LT1021 datasheet
VRefUNoiseTS  = [t timeseriesFromASD(VRefUNoiseASD, enbw, fSample, nSteps).Data]; 
% DAC Noise - AD660
DACNoiseASD = @(f) (0.125e-6.*f.^0) + 0.800e-6.*f.^-.5; %V/sqrt(Hz)
% AD660 20V span, int ref
DACNoiseTS  = [t timeseriesFromASD(DACNoiseASD, enbw, fSample, nSteps).Data];

%% Capacitance Bridge
% Tab. B-1
L  = 4.2e-3; %H
Cp = 320e-12; %F
C0 = 1.15e-12;  %Mance tab B1 ???
Q  = 200; %Mance tab B1

[brBlk, Zbr, f0br] = setCapBridge(mdl, 'capBridge', L, Cp, C0, Q);
% Bridge Impedance Thermal Noise - Mance 2012 eq 2.24
brRThNoise    = sqrt(4*kb*T*real(Zbr(f0br)));
brRThNoiseASD = @(f) brRThNoise.*f.^0;
brRThNoiseTS  = [t timeseriesFromASD(brRThNoiseASD, enbw, fSample, nSteps).Data];

%% Transimpedance Amplifier
% Tab. B-1
tiaCfb = 3.3e-12; %F
tiaRfb = 10e6;  %ohm
%brR = real(brZ(fDetection));
[tiaBlk, tiaZfb,tiaGs] = setTIA(mdl, 'TIAmp', Zbr(f0br), tiaCfb, tiaRfb);
%tiaR = real(tiaZfb(brf0));

% TIA Feedback Impedance Thermal Noise - Mance 2012 
tiaRfbThNoise    = 2*sqrt(4*kb*T*real(tiaZfb(f0br))); %V,sqrt(hz) eq 2.94
tiaRfbThNoiseASD = @(f) tiaRfbThNoise.*f.^0;
tiaRfbThNoiseTS  = [t timeseriesFromASD(tiaRfbThNoiseASD, enbw, fSample, nSteps).Data];

% TIA Current Noise - eq 2.95
tiaINoise    = 2*1.6e-15*real(tiaZfb(f0br));  % tab B1
tiaINoiseASD = @(f) tiaINoise.*f.^0;
tiaINoiseTS  = [t timeseriesFromASD(tiaINoiseASD, enbw, fSample, nSteps).Data];

% TIA Voltage Noise - eq 2.95
NGs = 1 + 2*abs(tiaZfb(f0br)/Zbr(f0br)); 
tiaUNoise    = 4.5e-9*NGs*sqrt(2);  % tab B1
tiaUNoiseASD = @(f) tiaUNoise.*f.^0;
tiaUNoiseTS  = [t timeseriesFromASD(tiaUNoiseASD, enbw, fSample, nSteps).Data]; 


%% AC amplifier
acGain = 45.4; % Ch. 2.8.5
% amplifier voltage noise
acUNoise    = 5.2e-9; %V/sqrt(Hz) tab 2.15 
acUNoiseASD = @(f) acUNoise.*f.^0;
acUNoiseTS  = [t timeseriesFromASD(acUNoiseASD, enbw, fSample, nSteps).Data]; 

%% Demodulator
% DC amp voltage noise 
dcampgain = 1.2; %calibrate so max +- 2.5V
DCampUNoiseASD = @(f) (0.03e-6.*f.^0) ; %V/sqrt(Hz) AD8629, mance
%DCampUNoiseASD = @(f) (14e-9.*f.^0) + 6.5e-9.*f.^-.7; %V/sqrt(Hz) OP97 (arbitrary)
DCampUNoiseTS  = [t timeseriesFromASD(DCampUNoiseASD, enbw, fSample, nSteps).Data]; 

%% ADC
% quantization noise
ADCqNoiseASD = @(f) (2.4e-6.*f.^0) ; %V/sqrt(Hz)  from eq 2.104
ADCqNoiseTS  = [t timeseriesFromASD(ADCqNoiseASD, enbw, fSample, nSteps).Data]; 

%% Noise Budget
% this uses the SimulinkNB toolbox.
% it doesn't run the simulation. Instead, it finds the transfer functions of the
% linearized model and plots the noise budged
[noises, sys] = nbFromSimulink(mdl, freq, 'dof', 'x');  
nb = nbGroupNoises(mdl, noises, sys);  
matlabNoisePlot(nb);  
  
%% Time domain simulation
mdlOutput = sim(mdl);    
% lpsd
Kdes = 100;
olap = 30;
Jdes = 1000;
Lmin = 1; 
order = 0;

[axx,fxx] = flpsd( 'ts', mdlOutput.simout.Data, 'fs', fSample, 'Kdes', Kdes, 'Lmin', Lmin, ...
        'Jdes', Jdes, 'win', 'hanning', 'olap', olap, 'order', order,...
        'scale', 'ASD', 'LTPDAmode',false);

%%
% finding the TF between displacement readout and input perturbation
io(1) = linio(strcat(mdl,'/C1'),1,'input');
io(2) = linio(strcat(mdl,'/Sum13'),1,'output');
readout2capTF = getTFresponse(mdl, io, fxx); % TF between readout and acc signal

loglog(fxx, axx./readout2capTF,'k','LineWidth',1);
xlabel('Frequency [Hz]');
ylabel('Measured capacitance [F]');