%% ACME - Simplified Capacitive Sensor Simulator
% All references are from Mance's 2012 thesis, unless otherwise noted.
% This model is calculated in the frequency range 1e-4 - 1Hz. Effects of the
% 100kHz injection signal are accounted for but the signal is not fully
% simulated.
% Only the dominant noise sources from the complete model are included
% Arthur Reis, 29.11.2022
clear;

mdl = 'capSensor_simplified';
load_system(mdl);
%% Simulation parameters
% frequency and time
stopTime = 1e4;
nSteps = 1e6;
tStep = stopTime/nSteps;
fSample = 1/(stopTime/nSteps);
t = (0:tStep:(stopTime-tStep))';
enbw = 1/stopTime;
freq = logspace(-4,0,100);
% Physical constants
eps0 = 8.86e-12; %F/m, vacuum permittivity
kb = 1.3806e-23; %boltzmann
T = 300; % K

%% Accelerometer Parameters
% TM, Ch.2 
TM_side = 0.046; %m,
mass = 2; %kg
nominal_gap = 0.004; % m 
area = TM_side^2; %m2

% Capacitive sensing
nominal_cap = 1.15e-12;% F 
Cmax = 0.12e-12; %Tab. B-1
fTypical = 1e-2; %Hz

%% Voltage Reference
Vd = 4.8; % Vpp, Ch. 2.6
fDetection = 1e5; %Hz 
capAttenuation = 0.123; %Eq 2.60, from ratio between injection
% and all other capacitances
% DAC Noise - AD660
DACNoiseASD = @(f) (0.125e-6.*f.^0) + 0.800e-6.*f.^-.5; %V/sqrt(Hz)
% AD660 20V span, int ref
DACNoiseTS  = [t timeseriesFromASD(DACNoiseASD, enbw, fSample, nSteps).Data];

%% Capacitance Bridge
% Tab. B-1
L  = 4.2e-3; %H
Cp = 320e-12; %F
C0 = 1.15e-12;  %Mance tab B1 ???
Q  = 200; %Mance tab B1

[brBlk, Zbr, f0br] = setCapBridge(mdl, 'capBridge', L, Cp, C0, Q);
% Bridge Impedance Thermal Noise - Mance 2012 eq 2.24
brRThNoise    = sqrt(4*kb*T*real(Zbr(f0br)));
brRThNoiseASD = @(f) brRThNoise.*f.^0;
brRThNoiseTS  = [t timeseriesFromASD(brRThNoiseASD, enbw, fSample, nSteps).Data];

%% Amplifier
% arbitrary gain for the simplified model, adjust if needed
% output should be max around +-2.5V
ampgain = 100;
%% Noise Budget
% this uses the SimulinkNB toolbox.
% it doesn't run the simulation. Instead, it finds the transfer functions of the
% linearized model and plots the noise budged
[noises, sys] = nbFromSimulink(mdl, freq, 'dof', 'x');  
nb = nbGroupNoises(mdl, noises, sys);  
  
%% Time domain simulation
mdlOutput = sim(mdl);    
% lpsd options
Kdes = 100;
olap = 30;
Jdes = 1000;
Lmin = 1; 
order = 0;
% lpsd 
[axx,fxx] = flpsd( 'ts', mdlOutput.simout.Data, 'fs', fSample, 'Kdes', Kdes, 'Lmin', Lmin, ...
        'Jdes', Jdes, 'win', 'hanning', 'olap', olap, 'order', order,...
        'scale', 'ASD', 'LTPDAmode',false);
%% Finding the transfer functions
% finding the transfer function between recorded signal and the signal we wish
% to measure
io(1) = linio(strcat(mdl,'/C1'),1,'input');
io(2) = linio(strcat(mdl,'/Amp'),1,'output');
readout2capTF = getTFresponse(mdl, io, fxx); % TF between readout and Cap1
sensorTFexp = fit(fxx,readout2capTF, 'poly2')

%% Expression for the total noise ASD
% assuming ASD = A/f + B, where B is the high freq limit
x = nb.sumNoise.f';
y = nb.sumNoise.asd';
sensorASDexp = fit(x,y, fittype(str2func(['@(a, x) a./x + ' num2str(y(end))])))

%from time domain simulation
xsim = fxx;
ysim = axx./readout2capTF;
sensorASDexpsim = fit(xsim,ysim, fittype(...
    str2func(['@(a, x) a./x + ' num2str(ysim(end))])))

%% Plots
matlabNoisePlot(nb);  
xlabel('Frequency [Hz]');
ylabel('Measured capacitance [F]');
loglog(freq, sensorASDexp(freq),'b.', 'displayName', 'Fit from lin model');
loglog(freq, sensorASDexpsim(freq),'k.','displayName', 'Fit  from sim');
loglog(fxx, axx./readout2capTF,'k-','LineWidth',1,'DisplayName', ...
    'ASD from time domain simulation');

%% Finding the transfer functions
% lpsd 
[axxs,fxxs] = flpsd( 'ts', mdlOutput.simout1.Data, 'fs', fSample, 'Kdes', Kdes, 'Lmin', Lmin, ...
        'Jdes', Jdes, 'win', 'hanning', 'olap', olap, 'order', order,...
        'scale', 'ASD', 'LTPDAmode',false);