% Electrostatic ACC Model 
% Following the Simplified Gravitational Reference Sensor model
% The S-GRS is a scaled-down version of the LPF GRS
% from: arXiv:2107.08545v1
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% last review: 03.02.2023
clear;
acme = AcmeSim('sgrs_acc',1e4+1,1e4+1);
%acme.openModel
%% Accelerometer Parameters 
%TM & EH Geometry
side = 0.03;
gap = 0.001;
elSides = [side, side];
density = 20099;

% % % If is drag-free:
% driverGain = 0;
% V_TM = 0.5;
% Vp = 0; %V polarizing 

% If is non-drag-free:1000
driverGain = -1;
V_TM = 0.5;
%Vp = 0; %V polarizing 

fCar = 1e5;

sat = Satellite;
%sat.altitude = 450; %edit here if needed. These are the defaults for GFO
% sat.mass = 470
% sat.Cd = 2.25
% sat.area = 0.9

TM   = acme.addTestMass('dblInt',side,gap,'density',density);
el1  = acme.addElectrode('electrode1', elSides, -(side/2 + gap), 0,    0, fCar);
el2  = acme.addElectrode('electrode2', elSides, -side/2        , 0, V_TM, fCar);
el3  = acme.addElectrode('electrode3', elSides,  side/2        , 0, V_TM, fCar);
el4  = acme.addElectrode('electrode4', elSides, +(side/2 + gap), 0,    0, fCar);
cap1 = acme.addCapacitor('planeCapacitor1', 'ElectrodePair', [el1, el2]);
cap2 = acme.addCapacitor('planeCapacitor2', 'ElectrodePair', [el3, el4]);

%CapNoiseASD = @(f) 1.854e-22 ./f + 5.0124e-19; %F/sqrt(hz)
CapNoiseASD = @(f) 51e-9.*f.^0; %F/sqrt(hz)
CapNoiseTS  = acme.noiseTS(CapNoiseASD);

CPDnoiseASD = @(f) cap1.force_from_CPD(10e-3)*f.^0; %worst case
CPDnoiseTS = acme.noiseTS(CPDnoiseASD);
Cf = 10e-12; %F

[a, v] = actuationRequirementSimplified(sat,TM,cap1);
ActuationNoiseASD = @(f) v * 1e-6*f.^0; %V/sqrt(hz)
ActuationNoiseTS  = acme.noiseTS(ActuationNoiseASD);
Vx0 = 2*v;
p = 1e-5; %Pa
gasThermalNoiseASD = @(f) gasBrownianNoise(p,acme.T,TM.l.x,TM.gap.x).*f.^0 + TM.mass*1.6e-15./sqrt(f);
gasThermalNoiseTS = acme.noiseTS(gasThermalNoiseASD);

% Control
Kp = 10;
Ki = 0.0;
Kd = 10;

%%
mdlOutput = sim(acme.ModelName);     % might not work for drag free
%%

vq = interp1(mdlOutput.simout.Time,mdlOutput.simout.Data,acme.Time);
ts = mdlOutput.simout.Data;%(10000:end);
%vq = interp1(out.simout.Time,mdlOutput.simout.Data,acme.Time);
%ts = out.simout.Data;%(10000:end);
[axx,fxx] = flpsd('ts', vq, acme.flpsdOpts{:});
io(2) = linio(strcat(acme.ModelName,'/BPF'),1,'output');
io(1) = linio(strcat(acme.ModelName,'/Gain1'),1,'input');
readout2accTF = getTFresponse(acme.ModelName, io, fxx); % 

%%
dof = 'x';  
[noises, sys] = nbFromSimulink(acme.ModelName, acme.Frequencies, 'dof', dof);  
nb = nbGroupNoises(acme.ModelName, noises, sys);  
matlabNoisePlot(nb);  
loglog(fxx,axx./readout2accTF)
xlim([1e-4 1e-1]);
ylim([1e-15 1e-8]);

%%
% [agt,fgt] = flpsd('ts', gasThermalNoiseTS(:,2), acme.flpsdOpts{:});
% io(2) = linio(strcat(acme.ModelName,'/Sum6'),1,'output');
% gast2accTF = getTFresponse(acme.ModelName, io, fgt); % 
% 
[acp,fcp] = flpsd('ts', CapNoiseTS(:,2), acme.flpsdOpts{:});
io(2) = linio(strcat(acme.ModelName,'/Sum'),1,'output');
cap2accTF = getTFresponse(acme.ModelName, io, fcp); % 

[aac,fac] = flpsd('ts', ActuationNoiseTS(:,2), acme.flpsdOpts{:});
io(2) = linio(strcat(acme.ModelName,'/Sum7'),1,'output');
act2accTF = getTFresponse(acme.ModelName, io, fgt); % 

hold on
%%
% loglog(fgt,agt./gast2accTF,'DisplayName','Gas Thermal');
matlabNoisePlot(nb);  
loglog(fxx,axx./readout2accTF,'DisplayName','Measurements')
loglog(fcp,acp./cap2accTF,'DisplayName','Cap');
loglog(fac,aac./act2accTF,'DisplayName','Act');

