% Electrostatic ACC Model 
% Following the Simplified Gravitational Reference Sensor model
% The S-GRS is a scaled-down version of the LPF GRS
% from: arXiv:2107.08545v1
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% last review: 30.01.2023
clear;
acme = AcmeSim('sgrs_acc',1e5,1e5);
%%sat parms
sat = Satellite();
% edit this as needed:
%sat.altitude = 450 km
%sat.mass     = 470 kg
%sat.Cd       = 2.25
%sat.area     = 0.9 m2
%% Accelerometer Parameters 
side = 0.03;
gap = 0.001;
elSides = [side, side];
density = 20099;

% % % If is drag-free:
% driverGain = 0;
% V_TM = 0.5;
% Vp = 0; %V polarizing 
% Vx0 = 0; %V rms

% If is non-drag-free:1000
Vp = 0; %V polarizing 
driverGain = -20;
V_TM = 0.5;
Vp = 0; %V polarizing 
Vx0 = 16; %V rms

fCar = 1e5;

el1 = acme.addElectrode('electrode1', elSides, -(side/2 + gap), 0,    0, fCar);
el2 = acme.addElectrode('electrode2', elSides, -side/2        , 0, V_TM, fCar);
el3 = acme.addElectrode('electrode3', elSides,  side/2        , 0, V_TM, fCar);
el4 = acme.addElectrode('electrode4', elSides, +(side/2 + gap), 0,    0, fCar);

TM = acme.addTestMass('dblInt',side,gap,'density',density);

cap1 = acme.addCapacitor('planeCapacitor1', 'ElectrodePair', [el1, el2]);
cap2 = acme.addCapacitor('planeCapacitor2', 'ElectrodePair', [el3, el4]);

CapNoiseASD = @(f) 51e-9.*f.^0; %F/sqrt(hz)
CapNoiseTS  = acme.noiseTS(CapNoiseASD);
[a, v] = actuationRequirementSimplified(sat,TM,cap1);
% Vx0 = 16; %V rms
%CPDnoiseASD = @(f) cap1.force_from_CPD(10e-3)*f.^0; %worst case
%CPDnoiseTS = acme.noiseTS(CPDnoiseASD);
Cf = 10e-12; %F

%%

ActuationNoiseASD = @(f) Vx0 * 1e-6*f.^0; %V/sqrt(hz)
ActuationNoiseTS  = acme.noiseTS(ActuationNoiseASD,'TS');

p = 1e-5; %Pa
gasThermalNoiseASD = @(f) gasBrownianNoise(p,acme.T,TM.l.x,TM.gap.x).*f.^0 + TM.mass*1.6e-15./sqrt(f);
gasThermalNoiseTS = acme.noiseTS(gasThermalNoiseASD);

% Control
Kp = 10;
Ki = 0.0;
Kd = 10;

%%
mdlOutput = sim(acme.ModelName);     % might not work for drag free
%%
vq = interp1(mdlOutput.simout.Time,mdlOutput.simout.Data,acme.Time);
ts = mdlOutput.simout.Data;%(10000:end);

[axx,fxx] = flpsd('ts', vq, acme.flpsdOpts{:});
io(2) = linio(strcat(acme.ModelName,'/BPF'),1,'output');
io(1) = linio(strcat(acme.ModelName,'/Gain1'),1,'input');
readout2accTF = getTFresponse(acme.ModelName, io, fxx); % 

%%
dof = 'x';  
[noises, sys] = nbFromSimulink(acme.ModelName, acme.Frequencies, 'dof', dof);  
nb = nbGroupNoises(acme.ModelName, noises, sys);  
matlabNoisePlot(nb);  
loglog(fxx,axx./readout2accTF)
xlim([1e-4 1e-1]);
ylim([1e-15 1e-8]);
