%% Example 02
% ACME Basics - Parametrization
% This example shows how a noise timeseries can be generated, also how you can
% parametrize your simulation
clear;

mdl = 'ex_02_noise_generation';
load_system(mdl);

%% Setting the TM and control parameters

mass = 0.08; %kg
gap = 0.0003; %m
Kp = 1e-4;
Ki = 1e-6;
Kd = 1e-2;
dblint = setDblIntTrans(mdl, 'dblInt', mass, gap); 

%% Define the input noise
% Will add a timeseries containing noise
% generated from expression for Amplitude Spectral Density

tslength = 1e4; 
dt = 1;
acme_dt_fixedstep = dt;
t = (0:dt:(tslength-dt))';
nSteps = length(t);
fSample = 1/dt;
enbw = 1/tslength;

% Consider the ASD:
noisefunc = @(f) (1e-10 * sqrt( (1e-3./f).^4 ./  ((1e-5./f).^4+1) + 1 + (f/1e-1).^4));

% define a "From Workspace" Block that contains the variable:
noise_ts = [t timeseriesFromASD(noisefunc, enbw, fSample, nSteps).Data];

%% Run
out = sim(mdl);  

%% Plotting the frequency response
% flpsd is the same lpsd algorithm from ltpda. It gives the ASD from a
% timeseries. 
enbw = 1;
Kdes = 100;
olap = 30;
Jdes = 1000;
Lmin = 1; 
order = 2;
ts = out.simout.Data; %output: position of the TM

[axx,fxx] = flpsd( 'ts', ts, 'fs', fSample, 'Kdes', Kdes, 'Lmin', Lmin, ...
        'Jdes', Jdes, 'win', 'hanning', 'olap', olap, 'order', order,...
        'scale', 'ASD', 'LTPDAmode',true);

[anoise,fnoise] = flpsd( 'ts', noise_ts(:,2), 'fs', fSample, 'Kdes', Kdes, 'Lmin', Lmin, ...
        'Jdes', Jdes, 'win', 'hanning', 'olap', olap, 'order', order,...
        'scale', 'ASD', 'LTPDAmode',true);

% finding the TF between displacement readout and input perturbation
io(1) = linio(strcat(mdl,'/From Workspace'),1,'input');
io(2) = linio(strcat(mdl,'/dblInt'),1,'output');
readout2accTF = getTFresponse(mdl, io, fxx); % TF between readout and acc signal

%% output comparison
loglog(fxx,noisefunc(fxx),fxx,axx./readout2accTF);
legend('input expression','simulated measurement');
xlabel('Frequency [Hz]');
xlim([1e-6 1]);
ylabel('Acceleration [m/s2]');

%% Parametrization
% To investigate what happens when the mass changes, for example:
mass_var = [0.04, 0.08, 0.16, 0.32];
amps = zeros(length(fxx),length(mass_var));
frqs = zeros(length(fxx),length(mass_var));
cals = zeros(length(fxx),length(mass_var));

for i=1:length(mass_var)
    setDblIntTrans(mdl, 'dblInt', mass_var(i), gap); 
    mass = mass_var(i);
    out = sim(mdl); % run the simulation for each mass value
    ts = out.simout.Data;
    [axx,fxx] = flpsd( 'ts', ts, 'fs', fSample, 'Kdes', Kdes, 'Lmin', Lmin, ...
        'Jdes', Jdes, 'win', 'hanning', 'olap', olap, 'order', order,...
        'scale', 'ASD', 'LTPDAmode',true); 
    frqs(:,i) = fxx; % save asd
    amps(:,i) = axx;
    cals(:,i) = getTFresponse(mdl, io, fxx);
end


%%
loglog(frqs, amps./cals);
legend(string(mass_var));
xlabel('Frequency [Hz]');
ylabel('Acceleration [m/s2]');
% nothing happens in an ideal, noiseless instrument