function freqs = createFrequencyArray(f1, f2, N, varargin)
%CREATEFREQUENCYARRAY returns either log or linspaced frequencies
%   Returns either log or linspaced frequencies, if linear, can be 
%   one or two sided, negative or positive first.
%
% Syntax: freqs = createFrequencyArray(f1,f2,N,['log']) 
%         freqs = createFrequencyArray(f1,_,N,'lin',[1]) 
%         freqs = createFrequencyArray(f1,_,N,'lin', 2, ['direct']) 
%       
% Inputs: f1 - frequency 1;
%         f2 - frequency 2;
%         N - number of points;
%         spacing - either 'lin' for linear or 'log' for base 10 logarithmic;
%         order - either 'direct', where the elements are sorted in crescent 
%         order, or 'reverse', where the positive elements come first;
%         sides - either 1, for only positives, or 2, to include negative
%         frequencies.
%
% Outputs: freqs - logspaced array between f1 and f2 with N points;
%          freqs - linspaced array between 0 f1 with N points;
%          freqs - linspaced array between -f1 and f1 with N points;
%          freqs - linspaced array between 0 to f1, then -f1 to 0 with N points; 
%
% See also: setInjectionBlock, createTimeseriesFromASD_ENBW
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Nov 2021; Last revision: 10.Nov.2021;

    
    if length(varargin)>=1
        spacing = varargin{1};
    else
        spacing = 'log';
    end
    
    if length(varargin)>=2
        sides = varargin{2};
    else
        sides = 1;
    end
    
    if length(varargin)>=3
        order = varargin{3};
    else
        order = 'direct';
    end
        
    if strcmp(spacing,'lin')
        f = linspace(0, f1, N);
        if sides == 1
            freqs = f;
            return;

        elseif sides == 2
            f = linspace(0,f1,N/2+1);
            if strcmp(order,'direct') % negative, positive
                if bitget(N,1) %even                  
                    freqs = [-f(end:-1:2),f];
                    return
                else
                    freqs = [-f(end-1:-1:2),f];
                    return;
                end
                    
            elseif strcmp(order,'reverse') % positive, negative
                if bitget(N,1) %even
                    freqs = [f,-f(end:-1:2)];
                    return
                else
                    freqs = [f,-f(end-1:-1:2)];
                    return;
                end    
            end   
        end
            
    elseif strcmp(spacing,'log')
        freqs = logspace(log10(f1),log10(f2),N);
        return;
    end

freqs = -1; %if reaches here, some error has occured.
end
        
