function [P,fArr,enbwcoef] = flpsd(varargin)
%% Compute PSD with logarithmic frequency axis
%
% code ported from LTPDA to bare MATLAB
%
% Example:
% y = randn(100000,1)*sqrt(3/2)*1e-3;
% [P,F] = flpsd( 'ts', y, 'fs', 3, 'scale','asd');
% loglog(F, P);
%
%
% INPUTS:
%   ts    - y-values of time-series
%   fs    - sample frequency in [Hz]
%   win   - window name, e.g. 'hanning'
%           see documentation of "getwin.m"
%   Order - order of detrending
%     -1  - no detrending
%     0   - subtract mean
%     1   - subtract linear fit
%     N   - subtract fit of polynomial, order N
%   Scale - Scaling of output. Choose from:
%     'AS'  - Amplitude (linear) Spectrum
%     'ASD' - Amplitude (linear) Spectral Density
%     'PS'  - Power Spectrum
%     'PSD' - Power Spectral Density [default]
%
%   Olap  - desired overlap percentage, value between 0..100
%           (default value of -1 means taken "optimum" for window function)
%   Kdes  - desired number of averages (default 100)
%   Lmin  - minimum segment length (default 1)
%   Jdes  - number of spectral frequencies to compute (default: 1000)
%
%   LTPDAmode - default: true
%               true means use LTPDA-like frequency vector and procedure
%
%               false: use strict appraoch from the paper (see reference
%               below). This mode is EXPERIMENTAL.
%
%   LTPDAbug  - 'true' will reproduce the LTPDA bug in Welford's averaging
%               algorithm, applies only if LTPDAmode = true
%               'false'
%
% OUTPUTS:
%   P    - spectral estimate in units requested by 'scale' parameter
%          Nx1
%   fArr - frequency values in [Hz]
%          Nx1
%   enbwcoeff - Equivalent Noise Bandwidth [Hz] for each frequency
%               Nx1
%
%
% -------------------------------------------------------------
%  References:  "Improved spectrum estimation from digitized time series
%                on a logarithmic frequency axis", Michael Troebs,
%                Gerhard Heinzel, Measurement 39 (2006) 120-129.
% --------------------
% Author: Arthur Reis; Vitali Mueller
% email: arthur.reis@aei.mpg.de
% --------------------------------------------------------------------
%% input processing
% parameters: ts, fs, Kdes, Lmin, Jdes, win, olap, order, scale
defaultts    = [];
defaultfs    = 0;
defaultKdes  = 100;
defaultLmin  = 1;
defaultJdes  = 1000;
defaultwin   = 'hanning';
defaultolap  = -1;
defaultorder = 0;
defaultscale = 'PSD';
expectedscales = {'AS','PS','ASD','PSD'};
defaultmode  = true;

% parameter parsing, required parms will throw error w/ defaults;
p = inputParser;
addParameter(p, 'ts',    defaultts, @(x) ~isempty(x));
addParameter(p, 'fs',    defaultfs, @(x) isnumeric(x) && x>0);
addParameter(p, 'Kdes',  defaultKdes, @(x) isnumeric(x) && x>0);
addParameter(p, 'Lmin',  defaultLmin, @(x) isnumeric(x));
addParameter(p, 'Jdes',  defaultJdes, @(x) isnumeric(x));
addParameter(p, 'win',   defaultwin);
addParameter(p, 'olap',  defaultolap, @(x) isnumeric(x) && (x<100 && x >=0 || x == -1) );
addParameter(p, 'order', defaultorder,...
    @(x) isscalar(x) && (x>=-1));
addParameter(p, 'scale', defaultscale,...
    @(x) any(validatestring(x,expectedscales)));
addParameter(p, 'LTPDAmode', defaultmode);
addParameter(p, 'LTPDAbug', false);

parse(p, varargin{:});

ts        = p.Results.ts;
if isrow(ts)
    ts = ts';
end
if size(ts,2) ~= 1
    error('input ts not Nx1 or 1xN');
end
fs        = p.Results.fs;
desAvg    = p.Results.Kdes;
Lmin      = p.Results.Lmin;
desFreqs  = p.Results.Jdes;
if desFreqs < 2
    error('Jdes should be >= 1')
end
wintype   = p.Results.win;
if p.Results.olap == -1
    [~, mw] = getwin(wintype,1);
    olap = mw.rov/100.0;
else
    olap = p.Results.olap/100.0;
end

order     = p.Results.order;
scale     = p.Results.scale;
ltpdamode = p.Results.LTPDAmode;
LTPDAbug  = p.Results.LTPDAbug;
%% ------ Set up some variables -------------------------------------------
entireData = ts;
nData = length(entireData);
fmin  = fs / nData; % * bmin;
fmax  = fs/2;
g = log(fmax) - log(fmin);
minAvgs  = 1;
maxFreqs = 2*desFreqs; %overestimate n. of fourier frequencies
ravg = fs/nData * (1 + (1-olap)*(double(desAvg)-1));
rmin = fs/nData * (1 + (1-olap)*(double(minAvgs)-1));

%% calculating freqs and resolution

bmin = 1;
%LTPDA Switch:
if ltpdamode
    % LTPDA version
    Ndata  = nData;
    Jdes   = desFreqs;
    Kdes   = desAvg;
    
    xov     = (1 - olap);
    fmin    = fs / Ndata * bmin;
    fmax    = fs/2;
    fresmin = fs / Ndata;
    freslim = fresmin * (1+xov*(Kdes-1));
    logfact = (Ndata/2)^(1/Jdes) - 1;
    
    f = [];
    r = [];
    b = [];
    L = [];
    K = [];
    
    fi = fmin;
    while fi < fmax
        
        fres = fi * logfact;
        if fres <= freslim
            fres = sqrt(fres*freslim);
        end
        if fres < fresmin
            fres = fresmin;
        end
        
        bin = fi/fres;
        if bin < bmin
            bin = bmin;
            fres = fi/bin;
        end
        
        dftlen = round(fs / fres);
        if dftlen > Ndata
            dftlen = Ndata;
        end
        if dftlen < Lmin
            dftlen = Lmin;
        end
        
        nseg = round((Ndata - dftlen) / (xov*dftlen) + 1);
        if nseg == 1
            dftlen = Ndata;
        end
        
        fres = fs / dftlen;
        bin  = fi / fres;
        
        % Store outputs
        f = [f fi]; %#ok<*AGROW>
        r = [r fres]; 
        b = [b bin];
        L = [L dftlen]; 
        K = [K nseg];
        
        fi = fi + fres;
        
    end
    segLength = L(:);
    fArr = f(:);
    binArr = b(:);
    P = zeros(size(fArr));
    enbwcoef = zeros(size(fArr));
    j = 1:length(f);
    nAvgs = K(:);
    
elseif ~ltpdamode
    % Implementation according to the paper (see reference in header)
    j = (0:1:maxFreqs-1);
    f = fmin*exp(g*(double(j)/(double(desFreqs)-1)));
    r_1p = f*g/(double(desFreqs)-1);
    r_2p = zeros(size(r_1p));
    
    segLength = zeros(1,maxFreqs);
    fRes   = zeros(1,maxFreqs);
    fArr   = zeros(1,maxFreqs);
    binArr = zeros(1,maxFreqs);
    jj = 1;
    fj = f(jj);
    while fj < fmax
        if r_1p(jj) >= ravg  % eq. 18 in the paper
            r_2p(jj) = r_1p(jj);
        elseif sqrt(r_1p(jj)*ravg) > rmin && r_1p(jj) < ravg % see errata
            r_2p(jj) = sqrt(r_1p(jj)*ravg);
        else
            r_2p(jj) = rmin;
        end
        segLength(jj) = round(fs/r_2p(jj));
        fRes(jj) = fs/double(segLength(jj));
        fArr(jj) = fj;
        binArr(jj) = fArr(jj)/fRes(jj);
        
        fj = fArr(jj) + fRes(jj);
        jj = jj+1;
    end
    %% -- crop ---
    segLength = segLength(1:jj-1)';
    fArr = fArr(1:jj-1)';
    binArr = binArr(1:jj-1)';
    P = zeros(1,jj-1)';
    enbwcoef = zeros(size(fArr));
    j = j(1:jj-1)';
    nAvgs = floor(double(nData - segLength)./(double(segLength).*(1-olap))+1);
    %A = (nAvgs == 1); why?
    %segLength(A) = nData;
end

%% -- loop over frequencies and segments
for jj = 1:length(j)
    ll = (0:segLength(jj)-1)';
    
    if nAvgs(jj) == 1
        segShift = 1;
    else
        segShift = (nData-segLength(jj))/(nAvgs(jj)-1);
    end
    
    Aj = zeros(1,nAvgs(jj));
    win = getwin(wintype,segLength(jj));
    S1 = sum(win);
    S2 = sum(win.^2);
    C_PS = 2/S1^2;
    C_PSD = 2/(fs*S2);
    
    % use same order in LTPDA (deviations close to machine precision)
    fourierFactor = exp(2*pi*1i*binArr(jj)/segLength(jj).*ll);
    detrendMat = [];
    detrendMatInv = [];
    
    for kk = 0:int32(nAvgs(jj))-1
        
        seg = entireData(kk*segShift+1:kk*segShift+segLength(jj));
        segAvg = sum(seg)/segLength(jj);
        
        %detrend options
        if order == -1 %no detrend
            weightedSeg = seg.*win;
        elseif order == 0
            weightedSeg = (seg - segAvg).*win;
        else
            if size(detrendMat,1) ~= numel(seg)
                % self-made polynomial fitting
                % matlab polyfit is too slow here
                detrendMat = ones(numel(seg),1);
                for iz=1:order
                    detrendMat(:,iz+1) = ((1:1:numel(seg))/numel(seg))'.^iz;
                end
                detrendMatInv=inv(detrendMat'*detrendMat)*detrendMat';  %#ok<MINV>
            end
            pf = detrendMatInv*seg;  % pf contains polynomial coefficients
            weightedSeg = (seg-detrendMat*pf).*win;
        end
        
        Aj(kk+1) = sum(weightedSeg .* fourierFactor);
    end
    
    if LTPDAbug
        % reproduce buggy Welford's algorithm that LTPDA is using
        tt = Aj.*conj(Aj);
        Mr = 0;
        for ii=1:length(tt)
            if ii==1
                Mr = tt(ii);
            else
                Mr = Mr + (tt(ii)-Mr)/(ii-1); % here is the off by one error
            end
        end
        % replace result with wrong result from LTPDA
        P(jj) = Mr; % replace result
    else
        % compute average
        P(jj) = sum(Aj.*conj(Aj))/(nAvgs(jj));
    end
    
    enbwcoef(jj) = fs*S2/S1^2;  
    scale = upper(scale);
    switch scale    
    case 'PSD'
        P(jj) = C_PSD*P(jj);
    case 'PS'
        P(jj) = C_PS*P(jj);
    case 'ASD'
        P(jj) = sqrt(C_PSD*P(jj));
    case 'AS'
        P(jj) = sqrt(C_PS*P(jj));
    otherwise
        error('undefined scale? %s',scale);
    end
end


if nargout == 0
    % create a basic plot
    figure; 
    plot(fArr, P)
    set(gca, 'yscale', 'log', 'xscale', 'log')
    xlabel('Fourier Frequency [Hz]')
    if strcmpi(scale, 'PS')
        ylabel('Signal PS [X^2]')
    elseif strcmpi(scale, 'PSD') 
        ylabel('Signal PSD [X^2/Hz]')
    elseif strcmpi(scale, 'AS')
        ylabel('Signal AS [X]')
    elseif strcmpi(scale, 'ASD')
        ylabel('Signal ASD [X/rtHz]')
    end

    clearvars('P'); % remove output, if no semicolon
end

end
