function [P, fArr, enbw, dev] = fpsd(varargin)
%%  Compute Power Spectral Density (PSD) of a time-series
%  
% Code has been ported from LTPDA to bare MATLAB
%
% Example:
% y = randn(100000,1)*sqrt(3/2)*1e-3;
% [P,F] = mypsd( 'ts', y, 'fs', 3, 'scale','asd');
% loglog(F, P);
%
%
% WOSA implements Welch's overlaped segmented averaging algorithm with
% segment detrending and variance estimation.
% 
% INPUTS:
%   ts    - y-values of time-series 
%   fs    - sample frequency in [Hz]
%   win   - window name, e.g. 'hanning'
%           see documentation of "getwin.m"
%   navs   - force numbers of averages
%               
%   Order - order of detrending
%     -1  - no detrending 
%     0   - subtract mean
%     1   - subtract linear fit
%     N   - subtract fit of polynomial, order N
%   Scale - Scaling of output. Choose from:
%     'AS'  - Amplitude (linear) Spectrum
%     'ASD' - Amplitude (linear) Spectral Density
%     'PS'  - Power Spectrum
%     'PSD' - Power Spectral Density [default]
%
%   Olap  - desired overlap percentage, value between 0..100
%           (default value of -1 means taken "optimum" for window function)
%   nfft   - number of samples in each fft [default: length of input data].
%
% OUTPUTS:
%   P    - spectral estimate in units requested by 'scale' parameter
%          Nx1
%   fArr - frequency values in [Hz]
%          Nx1
%   enbw - Equivalent Noise Bandwidth [Hz] 
%   dev  - standard deviation at each PSD sample (from averaging),
%          empty [] if no averaging was applied
%   
%
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% last rev: 08.02.2022
% --------------------------------------------------------------------

%% process inputs
defaultts    = [];
defaultfs    = 0;
defaultwin   = 'hanning';
defaultolap  = -1;
defaultnfft  = -1;
defaultnavs  = -1;
defaultmask  = [];
defaultorder = 0;
defaultscale = 'PSD';
expectedscales = {'AS','PS','ASD','PSD'};

% parameter parsing, required parms will throw error w/ defaults;
p = inputParser;
addParameter(p, 'ts',    defaultts, @(x) ~isempty(x));
addParameter(p, 'fs',    defaultfs, @(x) isnumeric(x) && x>0);
addParameter(p, 'win',   defaultwin);
addParameter(p, 'mask',  defaultmask);
addParameter(p, 'olap',  defaultolap);
addParameter(p, 'order', defaultorder,...
    @(x) isscalar(x) && (x>=-1));
addParameter(p, 'scale', defaultscale,...
    @(x) any(validatestring(x,expectedscales))); 
addParameter(p, 'nfft', defaultnfft);
addParameter(p, 'navs', defaultnavs);

parse(p, varargin{:});

ts        = p.Results.ts;
if isrow(ts)
   ts = ts'; 
end
if size(ts,2) ~= 1
    error('input ts not Nx1 or 1xN'); 
end
fs        = p.Results.fs;
wintype   = p.Results.win;
if p.Results.olap == -1
    [~, mw] = getwin(wintype,1);
    olap = mw.rov/100.0;
else
    olap = p.Results.olap/100.0;
end

mask      = p.Results.mask;
order     = p.Results.order;
scale     = p.Results.scale;
nfft      = p.Results.nfft;
navs      = p.Results.navs;
if nfft == -1 && navs < 1
    nfft = length(ts);
end %apply default
  
L = numel(ts);


if navs > 1
    if nfft ~= -1
       warning('navs and nfft specified. using nfft.'); 
    else
      % Compute the number of segments
      M = length(ts);
      L2 = round(M/(navs*(1-olap) + olap));
      %fprintf('Asked for navs = %d\n', navs);
      % Checks it will really obtain the correct answer.
      % This is needed to cope with the need to work with integers
      while fix((M-round(L2*olap))/(L2-round(L2*olap))) < navs
        L2 = L2 - 1;
      end
      %navs_actual = fix((M-round(L2*olap))/(L2-round(L2*olap)));
      %fprintf( 'Expect to get navs_actual = %d\n', navs_actual);
      if L2 > 0
        % Reset Nfft
        nfft = L2;
        %fprintf('reset navs to %d\n', fix(navs_actual));
      end
    end
end
  
winVals = getwin(wintype, nfft);
xolap        = round(olap*nfft); % Should this be round or floor?
nSegments = fix((L - xolap) ./ (nfft - xolap));

% Compute start and end indices of each segment
segmentStep = nfft - xolap;
segmentStarts = 1 : segmentStep : nSegments*segmentStep;
segmentEnds   = segmentStarts + nfft - 1;
  
if isempty(mask) %?
  mask = ones(1, nSegments);
end
if length(mask) ~= nSegments
  error('Please give a mask vector which is the same length as the number of segments (in this case %d)', nSegments);
end
  
% ensure we have logicals here
mask = logical(mask);
  
% filter the segments according to the mask
segmentStarts = segmentStarts(mask);
segmentEnds   = segmentEnds(mask);
nSegments     = numel(segmentStarts);
   
[Sxx, Svxx] = psdPeriodogram(ts, winVals, nSegments, ...
                segmentStarts, segmentEnds, order);
[Pxx, Pvxx] = scaleToPSD(Sxx, Svxx, nfft, fs);
% For the errors, the 1/nSegments factor should come after welchscale
% if we don't want to apply sqrt() to it. We correct for that here.
% It is only needed for 'asd','as' in psd/cpsd, the other cases go
% always through 'PSD'.
if (strcmpi(scale,'PSD') || strcmpi(scale,'PS'))
    dP = Pvxx;
elseif (strcmpi(scale,'ASD') || strcmpi(scale,'AS'))
    dP = Pvxx / nSegments;
else
    error('### Unknown scale')
end
  
% Compute frequencies
freqs = psdfreqvec('npts', nfft, 'Fs', fs, 'Range', 'half').';
 
% Scale to required units
[Pxx, dP, enbw] = welchscale(Pxx, dP, winVals, fs, scale);
  
if nSegments == 1
    dev = [];
else
    dev = sqrt(dP);
end
  
% Set outputs
P = Pxx;
fArr = freqs';
%     enbw = enbw; %#ok<ASGSL>
%     dev = dev; %#ok<ASGSL>

if nargout == 0
    % create a basic plot
    figure; 
    plot(freqs', Pxx)
    set(gca, 'yscale', 'log', 'xscale', 'log')
    xlabel('Fourier Frequency [Hz]')
    if strcmpi(scale, 'PS')
        ylabel('Signal PS [X^2]')
    elseif strcmpi(scale, 'PSD') 
        ylabel('Signal PSD [X^2/Hz]')
    elseif strcmpi(scale, 'AS')
        ylabel('Signal AS [X]')
    elseif strcmpi(scale, 'ASD')
        ylabel('Signal ASD [X/rtHz]')
    end

    clearvars('P'); % remove output, if no semicolon
end
end

% scale averaged periodogram to PSD
function [Pxx, Pvxx] = scaleToPSD(Sxx, Svxx, nfft, fs)
  
  % Take 1-sided spectrum which means we double the power in the
  % appropriate bins
  if rem(nfft,2)
    indices = 1:(nfft+1)/2;  % ODD
    Sxx1sided = Sxx(indices,:);
    % double the power except for the DC bin
    Sxx = [Sxx1sided(1,:); 2*Sxx1sided(2:end,:)];  
    if ~isempty(Svxx)
      Svxx1sided = Svxx(indices,:);
      Svxx = [Svxx1sided(1,:); 4*Svxx1sided(2:end,:)];
    end
  else
    indices = 1:nfft/2+1;    % EVEN
    Sxx1sided = Sxx(indices,:);
    % Double power except the DC bin and the Nyquist bin
    Sxx = [Sxx1sided(1,:); 2*Sxx1sided(2:end-1,:); Sxx1sided(end,:)];
    if ~isempty(Svxx)
      Svxx1sided = Svxx(indices,:); % Take only [0,pi] or [0,pi)
      Svxx = [Svxx1sided(1,:); 4*Svxx1sided(2:end-1,:); Svxx1sided(end,:)];
    end
 end

  % Now scale to PSD
  Pxx   = Sxx ./ fs;
  Pvxx  = Svxx ./ fs^2;
end

% compute psd
function [Sxx, Svxx] = psdPeriodogram(x, winVals, nSegments, segmentStarts, segmentEnds, order)
    Mnxx = 0; 
    Mn2xx = 0;
    nfft = segmentEnds(1) - segmentStarts(1) + 1;
    detrendMat = [];
    detrendMatInv = [];    
    % Loop over the segments
    for ii = 1:nSegments
        seg = x(segmentStarts(ii):segmentEnds(ii));     
        if order == -1 % 
            ;    %#ok<NOSEMI>
        elseif order == 0 
            seg = (seg - mean(seg));
        else
            if size(detrendMat,1) ~= numel(seg)
                detrendMat = ones(numel(seg),1);
                for iz=1:order
                    detrendMat(:,iz+1) = ((1:1:numel(seg))/numel(seg))'.^iz;
                end
                detrendMatInv=inv(detrendMat'*detrendMat)*detrendMat'; %#ok<MINV>
            end
            pf = detrendMatInv*seg;
            seg = (seg-detrendMat*pf);
        end 
        % Compute periodogram
        Sxxk = wosa_periodogram(seg, [], winVals, nfft);
        % Welford's algorithm for updating mean and variance
        if ii == 1
            Mnxx = Sxxk;
        else
            Qxx = Sxxk - Mnxx;
            Mnxx = Mnxx + Qxx/ii;
            Mn2xx = Mn2xx + Qxx .* (Sxxk - Mnxx);
        end
    end
  Sxx = Mnxx;
  if nSegments == 1
    Svxx = [];
  else
    Svxx = Mn2xx/(nSegments-1)/nSegments;
  end
  
end

function Sxx = wosa_periodogram(x, ~, win, nfft)
  
  xwin = x .* win;
  X = fft(xwin, nfft);
  K = win' * win;
  Sxx = X .* conj(X) / K;

end

function [yy, dyy, enbw] = welchscale(xx, dxx, win, fs, norm)
  
  S1   = sum(win);
  S2   = sum(win.^2);
  enbw = fs * S2 / (S1*S1);
  
  if isempty(norm)
    norm = 'None';
  end
  switch lower(norm)
    case 'asd'
      yy = sqrt(xx);
      if isempty(dxx)
        dyy = dxx;
      else
        dyy = 1./2./sqrt(xx) .* dxx;
      end
      
    case 'psd'
      yy = xx;
      dyy = dxx;
      
    case 'as'
      yy = sqrt(xx * enbw);
      if isempty(dxx)
        dyy = dxx;
      else
        dyy = 1./2./sqrt(xx) .* dxx * enbw;
      end
      
    case 'ps'
      yy = xx * enbw;
      dyy = dxx * enbw;
      
    case 'none'
      yy = xx;
      dyy = dxx;
      
    otherwise
      error('Unknown normalisation');
  end  
end