function varargout = getwin(varargin) 
%% GETWIN - return an array of length N of the window function.
%
% Uses the implementation of window functions from LTPDA and returns a simple
% array of values.
%
% Example:
% [winvals, winstruct] = getwin('hanning', N)
% [winstruct] = getwin('hanning')
%
%
% Inputs:
%      winname - name of the window function, string, see list below
%      N - length of the array, integer;
%
% Outputs:
%      winvals - window values Nx1
%      winstruct  - struct containing informtion on the window function
%
% -----------------------------------------------------------------------
% Reference:  Heinzel, G., Rüdiger, A., & Schilling, R. (2002). Spectrum and 
% spectral density estimation by the Discrete Fourier transform (DFT), 
% including a comprehensive list of window functions and some new 
% flat-top windows., http://hdl.handle.net/11858/00-001M-0000-0013-557A-5
% -----------------------------------------------------------------------
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Nov 2021; Last revision: 28.Jan.2022;
% ---------------------------
% List of available window functions:
% 'Rectangular'
% 'Welch'
% 'Bartlett'
% 'Hanning'
% 'Hamming'
% 'Nuttall3'
% 'Nuttall4'
% 'Nuttall3a'
% 'Nuttall3b'
% 'Nuttall4a'
% 'Nuttall4b'
% 'Nuttall4c'
% 'BH92'
% 'SFT3F'
% 'SFT3M'
% 'FTNI'
% 'SFT4F'
% 'SFT5F'
% 'SFT4M'
% 'FTHP'
% 'HFT70'
% 'FTSRS'
% 'SFT5M'
% 'HFT90D'
% 'HFT95'
% 'HFT116D'
% 'HFT144D'
% 'HFT169D'
% 'HFT196D'
% 'HFT223D'
% 'HFT248D'
% 'Kaiser'
% 'levelledHanning'

    
    window.type        = ''; % name of window object
    window.alpha       = []; % alpha parameter for various window functions
    window.psll        = []; % peak sidelobe level
    window.rov         = []; % recommended overlap
    window.nenbw       = []; % normalised equivalent noise bandwidth
    window.w3db        = []; % 3 dB bandwidth in bins
    window.flatness    = []; % window flatness
    window.levelorder  = []; % levelling coefficient
    window.skip        = []; %#ok<*STRNU> % number of bins to skip        
    % 
    if ~exist('win_hanning.m','file')
        error('you need to add facade repository or "windows" sub-folder to PATH');
    end
    if nargout == 1 && nargin == 1
        if isa(varargin{1},'struct')
            winname = lower(varargin{1}.type);
            N = varargin{1}.len; %#ok<NASGU>
            window = eval(strcat('win_',winname,'(window, ''define'', N)'));  %#ok<NASGU>
            varargout{1} = eval(strcat('win_',winname,'(window, ''build'', N)'))'; 
        else
            % if only the name is specified, return struct without evaluating
            %return only struct
            N = -1;  %#ok<NASGU>
            winname = lower(varargin{1});
            window = eval(strcat('win_',winname,'(window, ''define'', N)'));
            varargout{1} = window;
        end
    elseif nargout == 1 && nargin == 2
        % return only values
        winname = lower(varargin{1});
        N = varargin{2}; %#ok<NASGU>
        window = eval(strcat('win_',winname,'(window, ''define'', N)'));  %#ok<NASGU>
        window_values = eval(strcat('win_',winname,'(window, ''build'', N)'))';
        varargout{1} = window_values;
    elseif nargout == 2 && nargin == 2
        % return values and struct
        winname = lower(varargin{1});
        N = varargin{2};  %#ok<NASGU>
        window = eval(strcat('win_',winname,'(window, ''define'', N)'));
        varargout{1} = eval(strcat('win_',winname,'(window, ''build'', N)'))';
        varargout{2} = window;       
    end   
end

