function TFresponse = getTSfromTF(varargin)
%GETTFRESPONSE returns the response of the transfer function
%of the linearized model.
%   Returns the response of the transfer function of the linearized
%   model. First, linearizes the model around the given input and output. Then,
%   it gets the frequency response of the linear TF. 
% Syntax: TFresponse = getTFresponse(mdl, io, freqs)
%       
% Inputs: mdl - simulink model, string;
%         io - list of IO elements;
%         u - ts
%         t - time
%         mode - direct/inverted
%
% Outputs: TFresponse - array containing the response, 
%             same size as the frequencies;
%
% See also: frd, linearize, linio, getInvertedTFresponse
%
% Author: Arthur Reis
% email: arthur.reis@aei.mpg.de
% Nov 2021; Last revision: Aug.2022;  

defaultMode = "direct";

p = inputParser;

addRequired(p, 'mdl');
addRequired(p, 'io');
addRequired(p, 'u');
addRequired(p, 't');
addParameter(p, "mode", defaultMode);
parse(p, varargin{:});            

mdl  = p.Results.mdl;
io   = p.Results.io;
u    = p.Results.u;
t    = p.Results.t;
mode = p.Results.mode;

linearizedModel = linearize(mdl,io);
if strcmp(mode,defaultMode)
    linearizedModel = linearize(mdl,io);
elseif strcmp(mode,"inverted")
    linearizedModel = linearize(mdl,io);
end
lsim(linearizedModel,u,t);
linMdlResponse = frd(linearizedModel, freqs, 'unit', 'Hz');
TFresponse = squeeze(abs(linMdlResponse.ResponseData));
end

