function [P,fArr] = logpsd(varargin)
%% Adapted LTPDA syntax
% INPUTS (compared with LTPDA):
%   as    - array of analysis objects [REMOVED] 
%   pl    - parameter list [REMOVED]
%   ts    - timeseries [ADDED]
%   fs    - sample freqs
%   Kdes  - desired number of averages (default 100)
%   Lmin  - minimum segment length (default 0)
%   Jdes  - number of spectral frequencies to compute (default fs/2)
%   win   - a specwin window object. [REMOVED]
%   win   - a struct. See GETWIN. [ADDED]
%   Olap  - desired overlap percentage (default: taken from window)
%   Order - order of detrending
%     -1  - no detrending 
%     0   - subtract mean
%     1   - subtract linear fit
%     N   - subtract fit of polynomial, order N
%   Scale - Scaling of output. Choose from:
%     AS  - Amplitude (linear) Spectrum%     ASD - Amplitude (linear) Spectral Density
%     PS  - Power Spectrum
%     PSD - Power Spectral Density [default]
%
% OUTPUTS:
%   [REMOVED] bs  - array of analysis objects, one for each input
%   ps    - spectral estimate
%   

%% input processing
% parameters: ts, fs, Kdes, Lmin, Jdes, win, olap, order, scale
defaultts    = [];
defaultfs    = 0;
defaultKdes  = 100;
defaultLmin  = 1;
defaultJdes  = 0; 
defaultwin   = '';
defaultolap  = -1;
defaultorder = 0;
defaultscale = 'PSD';
expectedscales = {'AS','PS','ASD','PSD'};
defaultmode  = false;

% parameter parsing, required parms will throw error w/ defaults;
p = inputParser;
addParameter(p, 'ts',    defaultts, @(x) ~isempty(x));
addParameter(p, 'fs',    defaultfs, @(x) isnumeric(x) && x>0);
addParameter(p, 'Kdes',  defaultKdes, @(x) isnumeric(x) && x>0);  
addParameter(p, 'Lmin',  defaultLmin, @(x) isnumeric(x));
addParameter(p, 'Jdes',  defaultJdes, @(x) isnumeric(x));
addParameter(p, 'win',   defaultwin);
addParameter(p, 'olap',  defaultolap);
addParameter(p, 'order', defaultorder,...
    @(x) isscalar(x) && (x>=-1));
addParameter(p, 'scale', defaultscale,...
    @(x) any(validatestring(x,expectedscales))); 
addParameter(p, 'LTPDAmode', defaultmode); 

parse(p, varargin{:});

ts        = p.Results.ts;
fs        = p.Results.fs;
desAvg    = p.Results.Kdes;
Lmin      = p.Results.Lmin;
desFreqs  = p.Results.Jdes; %correct default fs/2
if desFreqs == 0
    desFreqs = fs/2;
end
wintype   = p.Results.win;
if p.Results.olap == -1
    [~, mw] = getwin(wintype,1);
    olap = mw.rov/100.0;
else
    olap = p.Results.olap/100.0;
end

order     = p.Results.order;
scale     = p.Results.scale;
ltpdamode = p.Results.LTPDAmode;

%% ------ Set up some variables -------------------------------------------
  entireData = ts;
  nData = length(entireData);
  fmin  = fs / nData; % * bmin;
  fmax  = fs/2;
  g = log(fmax) - log(fmin);
  minAvgs  = 1;
  maxFreqs = 2*desFreqs; %overestimate n. of fourier frequencies
  ravg = fs/nData * (1 + (1-olap)*(double(desAvg)-1));
  rmin = fs/nData * (1 + (1-olap)*(double(minAvgs)-1));
  xov  = (1 - olap);

%% calculating freqs and resolution

bmin = 1;
%LTPDA Switch:
if ltpdamode
% LTPDA version
  Ndata  = nData;
  Jdes   = desFreqs;
  Kdes   = desAvg;

  xov     = (1 - olap);
  fmin    = fs / Ndata * bmin;
  fmax    = fs/2;
  fresmin = fs / Ndata;
  freslim = fresmin * (1+xov*(Kdes-1));
  logfact = (Ndata/2)^(1/Jdes) - 1;

  f = [];
  r = [];
  b = [];
  L = [];
  K = [];

  fi = fmin;
  while fi < fmax

    fres = fi * logfact;
    if fres <= freslim
      fres = sqrt(fres*freslim);
    end
    if fres < fresmin
      fres = fresmin;
    end

    bin = fi/fres;
    if bin < bmin
      bin = bmin;
      fres = fi/bin;
    end

    dftlen = round(fs / fres);
    if dftlen > Ndata
      dftlen = Ndata;
    end
    if dftlen < Lmin
      dftlen = Lmin;
    end

    nseg = round((Ndata - dftlen) / (xov*dftlen) + 1);
    if nseg == 1
      dftlen = Ndata;
    end

    fres = fs / dftlen;
    bin  = fi / fres;

    % Store outputs
    f = [f fi];
    r = [r fres];
    b = [b bin];
    L = [L dftlen];
    K = [K nseg];

    fi = fi + fres;

  end 
  segLength = L(:);
  fArr = f(:);
  binArr = b(:);
  P = zeros(size(fArr));
  j = 1:length(f);
  nAvgs = K(:);
  
elseif ~ltpdamode  
% Gerhard's paper version:
  j = (0:1:maxFreqs-1);
  f = fmin*exp(g*(double(j)/(double(desFreqs)-1)));
  r_1p = f*g/(double(desFreqs)-1);
  r_2p = zeros(size(r_1p));
  
  segLength = zeros(1,maxFreqs);
  fRes   = zeros(1,maxFreqs);
  fArr   = zeros(1,maxFreqs);
  binArr = zeros(1,maxFreqs);
  jj = 1;
  fj = f(jj);
    while fj < fmax
        if r_1p(jj) >= ravg  % see errata
            r_2p(jj) = r_1p(jj); 
        elseif sqrt(r_1p(jj)*ravg) > rmin && r_1p(jj) < ravg
            r_2p(jj) = sqrt(r_1p(jj)*ravg);
        else
            r_2p(jj) = rmin;
        end
   
  
        segLength(jj) = round(fs/r_2p(jj));
        fRes(jj) = fs/double(segLength(jj));
        fArr(jj) = fj;
        binArr(jj) = fArr(jj)/fRes(jj);
    
        fj = fArr(jj) + fRes(jj);
        jj = jj+1;
    end
  % -- crop ---  
  segLength = segLength(1:jj-1)';
  fArr = fArr(1:jj-1)';
  binArr = binArr(1:jj-1)';
  P = zeros(1,jj-1)';
  j = j(1:jj-1)';
  nAvgs = floor(double(nData - segLength)./(double(segLength).*(1-olap))+1);  
  
end


%% -- loop over frequencies and segments  
for jj = 1:length(j)
    ll = (0:segLength(jj)-1)'; 
    segShift = (1-olap)*segLength(jj);
    Aj = zeros(1,nAvgs(jj));
    win = getwin(wintype,segLength(jj));
    S1 = sum(win);
    S2 = sum(win.^2);
    C_PS = 2/S1^2;
    C_PSD = 2/(fs*S2);
    
    fourierFactor = exp(-2*pi*1i*binArr(jj)*ll/segLength(jj));     
    detrendMat = [];
    detrendMatInv = [];

    for kk = 0:int32(nAvgs(jj))-1
        
        if (kk*segShift+segLength(jj)) <= length(entireData)
            seg = entireData(kk*segShift+1:kk*segShift+segLength(jj));
            segAvg = sum(seg)/segLength(jj);
        else %out of range
            seg = entireData(kk*segShift+1:end);
            segLenEnd = length(seg);
            segAvg = sum(seg)/segLenEnd;
            win = getwin(wintype,segLenEnd); %get shorter window
            llEnd = (0:segLenEnd-1)'; 
            fourierFactor = exp(-2*pi*1i*binArr(jj)*llEnd/segLenEnd);
        end
        
        %detrend options
        if order == -1 %no detrend
                weightedSeg = seg.*win;    
        elseif order == 0 
                weightedSeg = (seg - segAvg).*win;
        else
            if any(size(detrendMat,1) ~= [numel(seg)])
                detrendMat = ones(numel(seg),1);
                    for iz=1:order
                        detrendMat(:,iz+1) = ([1:1:numel(seg)]/numel(seg))'.^iz;
                    end
                        detrendMatInv=inv(detrendMat'*detrendMat)*detrendMat';
            end
            pf = detrendMatInv*seg;
            weightedSeg = (seg-detrendMat*pf).*win;
        end
        Aj(kk+1) = sum(weightedSeg .* fourierFactor);
    end
    
    P(jj) = sum(Aj.*conj(Aj))/nAvgs(jj);
      
    enbwcoef = fs*S2/S1^2;  
    switch scale    
    case 'PSD'
        P(jj) = C_PSD*P(jj);
    case 'PS'
        P(jj) = C_PS*P(jj);
    case 'ASD'
        P(jj) = sqrt(C_PSD*P(jj));
    case 'AS'
        P(jj) = sqrt(C_PS*P(jj));
    end
end
    
end