function ts = timeseriesFromASD(varargin)
%TIMESERIESFROMASD returns a timeseries from the ASD of a noise
%   Returns a timeseries from the amplitude spectral density of a noise.
%   The output time series will have the ASD, and random phase.
%
% Syntax: ts = timeseriesFromASD(ASDfunc, enbw, fSample, N)
%       
% Inputs: ASDfunc - function handle where the expression for the ASD is
%         defined;
%         enbw    - equivalent noise bandwith = 1/total_observation_time, in s;
%         fSample - sampling frequency, in Hz;
%         N       - number of samples;
%
% Outputs: ts - timeseries containing the noise;
%
% See also: createFrequencyArray, setInjectionBlock 
%
% Author: Arthur Reis, Alexey Kupryianov
% email: arthur.reis@aei.mpg.de, 
% Nov 2021; Last revision: 11.Jul.2022;    

p = inputParser;

addRequired(p, 'ASDfunc');
addRequired(p, 'enbw'   , @(x) isnumeric(x) && isscalar(x));
addRequired(p, 'fSample', @(x) isnumeric(x) && isscalar(x));
addRequired(p, 'N'      , @(x) isnumeric(x) && isscalar(x));

parse(p, varargin{:});            

ASDfunc = p.Results.ASDfunc;
enbw    = p.Results.enbw;
fSample = p.Results.fSample;
N       = double(p.Results.N);
if rem(N,2)==0 %if N is even
    halfF = fSample/2*linspace(0,1,N/2+1); % single-sided positive frequency
    fSeries= [halfF, -(halfF(end-1:-1:2))];
else %if N is odd
    halfF = fSample/2*linspace(0,1,N/2+1);
    fSeries= [halfF, -(halfF(end-1:-1:1))];
end
%halfF = fSample/2*linspace(0,1,N/2+1);  % single-sided positive frequency
%fSeries= [halfF, -(halfF(end-1:-1:2))];
ASD = ASDfunc(fSeries);
PSD_dblsided = (ASD).^2/2;
%     if length(PSD_dblsided ) == 1
%         PSD_dblsided = PSD_dblsided *ones(size(freqs));
%     end
PSD_dblsided(fSeries == 0) = 0;
U_f = sqrt(PSD_dblsided').*sqrt(enbw/2);
phi = rand(size(U_f))*2*pi;
phi(fSeries==0) = 0; 
freq_domain = U_f.*exp(phi.*1i)*numel(U_f)*sqrt(2);
t_amplitude = ifft(freq_domain,'symmetric');
    
%ts = timeseries(t_amplitude,linspace(0,2/fSample,length(t_amplitude)));  
timevector = 1:1/fSample:N;
    ts = timeseries(t_amplitude,timevector);  
end